@@ -36,7 +36,7 @@
 #include "mapserver.h"
 #include "cgiutil.h"
 
-
+#include "cpl_conv.h"
 
 #define LF 10
 #define CR 13
@@ -46,6 +46,8 @@ int readPostBody( cgiRequestObj *request, char **data )
   size_t data_max, data_len;
   int chunk_size;
 
+  (void)request;
+
   msIO_needBinaryStdin();
 
   /* -------------------------------------------------------------------- */
@@ -55,23 +57,26 @@ int readPostBody( cgiRequestObj *request, char **data )
     data_max = (size_t) atoi(getenv("CONTENT_LENGTH"));
     /* Test for suspicious CONTENT_LENGTH (negative value or SIZE_MAX) */
     if( data_max >= SIZE_MAX ) {
-      msIO_setHeader("Content-Type","text/html");
-      msIO_sendHeaders();
-      msIO_printf("Suspicious Content-Length.\n");
+      // msIO_setHeader("Content-Type","text/html");
+      // msIO_sendHeaders();
+      // msIO_printf("Suspicious Content-Length.\n");
+      msSetError(MS_WEBERR, "Suspicious Content-Length.", "readPostBody()");
       return MS_FAILURE;
     }
     *data = (char *) malloc(data_max+1);
     if( *data == NULL ) {
-      msIO_setHeader("Content-Type","text/html");
-      msIO_sendHeaders();
-      msIO_printf("malloc() failed, Content-Length: %u unreasonably large?\n", (unsigned int)data_max );
+      // msIO_setHeader("Content-Type","text/html");
+      // msIO_sendHeaders();
+      // msIO_printf("malloc() failed, Content-Length: %u unreasonably large?\n", (unsigned int)data_max );
+      msSetError(MS_WEBERR, "malloc() failed, Content-Length: %u unreasonably large?", "readPostBody()", (unsigned int)data_max);
       return MS_FAILURE;
     }
 
-    if( (int) msIO_fread(*data, 1, data_max, stdin) < data_max ) {
-      msIO_setHeader("Content-Type","text/html");
-      msIO_sendHeaders();
-      msIO_printf("POST body is short\n");
+    if( (int) msIO_fread(*data, 1, data_max, stdin) < (int) data_max ) {
+      // msIO_setHeader("Content-Type","text/html");
+      // msIO_sendHeaders();
+      // msIO_printf("POST body is short\n");
+      msSetError(MS_WEBERR, "POST body is short.", "readPostBody()");
       return MS_FAILURE;
     }
 
@@ -86,6 +91,7 @@ int readPostBody( cgiRequestObj *request, char **data )
   data_max = DATA_ALLOC_SIZE;
   data_len = 0;
   *data = (char *) msSmallMalloc(data_max+1);
+  (*data)[data_max] = '\0';
 
   while( (chunk_size = msIO_fread( *data + data_len, 1, data_max-data_len, stdin )) > 0 ) {
     data_len += chunk_size;
@@ -93,9 +99,10 @@ int readPostBody( cgiRequestObj *request, char **data )
     if( data_len == data_max ) {
       /* Realloc buffer, making sure we check for possible size_t overflow */
       if ( data_max > SIZE_MAX - (DATA_ALLOC_SIZE+1) ) {
-        msIO_setHeader("Content-Type","text/html");
-        msIO_sendHeaders();
-        msIO_printf("Possible size_t overflow, cannot reallocate input buffer, POST body too large?\n" );
+        // msIO_setHeader("Content-Type","text/html");
+        // msIO_sendHeaders();
+        // msIO_printf("Possible size_t overflow, cannot reallocate input buffer, POST body too large?\n" );
+	msSetError(MS_WEBERR, "Possible size_t overflow, cannot reallocate input buffer, POST body too large?", "readPostBody()");
         return MS_FAILURE;
       }
 
@@ -110,6 +117,7 @@ int readPostBody( cgiRequestObj *request, char **data )
 
 static char* msGetEnv(const char *name, void* thread_context)
 {
+  (void)thread_context;
   return getenv(name);
 }
 
@@ -136,18 +144,23 @@ int loadParams(cgiRequestObj *request,
 
   debuglevel = (int)msGetGlobalDebugLevel();
 
-  if(strcmp(getenv2("REQUEST_METHOD", thread_context),"POST") == 0) { /* we've got a post from a form */
+  if(strcmp(getenv2("REQUEST_METHOD", thread_context),"POST") == 0 && CPLGetConfigOption("MS_NO_POST", NULL) == NULL) { /* we've got a post from a form */
     char *post_data;
     int data_len;
     request->type = MS_POST_REQUEST;
 
-    s = getenv2("CONTENT_TYPE", thread_context);
-    if (s != NULL)
-      request->contenttype = msStrdup(s);
-    /* we've to set default Content-Type which is
-     * application/octet-stream according to
-     * W3 RFC 2626 section 7.2.1 */
-    else request->contenttype = msStrdup("application/octet-stream");
+    if (request->contenttype == NULL){
+        s = getenv2("CONTENT_TYPE", thread_context);
+        if (s != NULL){
+          request->contenttype = msStrdup(s);
+        }
+        else {
+            /* we've to set default Content-Type which is
+             * application/octet-stream according to
+             * W3 RFC 2626 section 7.2.1 */
+            request->contenttype = msStrdup("application/octet-stream");
+        }
+    }
 
     if (raw_post_data) {
       post_data = msStrdup(raw_post_data);
@@ -207,9 +220,10 @@ int loadParams(cgiRequestObj *request,
 
       s = getenv2("QUERY_STRING", thread_context);
       if(s == NULL) {
-        msIO_setHeader("Content-Type","text/html");
-        msIO_sendHeaders();
-        msIO_printf("No query information to decode. QUERY_STRING not set.\n");
+        // msIO_setHeader("Content-Type","text/html");
+        // msIO_sendHeaders();
+        // msIO_printf("No query information to decode. QUERY_STRING not set.\n");
+	msSetError(MS_WEBERR, "No query information to decode. QUERY_STRING not set.", "loadParams()");
         return -1;
       }
 
@@ -217,9 +231,10 @@ int loadParams(cgiRequestObj *request,
         msDebug("loadParams() QUERY_STRING: %s\n", s);
 
       if(strlen(s)==0) {
-        msIO_setHeader("Content-Type","text/html");
-        msIO_sendHeaders();
-        msIO_printf("No query information to decode. QUERY_STRING is set, but empty.\n");
+        // msIO_setHeader("Content-Type","text/html");
+        // msIO_sendHeaders();
+        // msIO_printf("No query information to decode. QUERY_STRING is set, but empty.\n");
+	msSetError(MS_WEBERR, "No query information to decode. QUERY_STRING is set, but empty.", "loadParams()");
         return -1;
       }
 
@@ -238,9 +253,10 @@ int loadParams(cgiRequestObj *request,
         m++;
       }
     } else {
-      msIO_setHeader("Content-Type","text/html");
-      msIO_sendHeaders();
-      msIO_printf("This script should be referenced with a METHOD of GET or METHOD of POST.\n");
+      // msIO_setHeader("Content-Type","text/html");
+      // msIO_sendHeaders();
+      // msIO_printf("This script should be referenced with a METHOD of GET or METHOD of POST.\n");
+      msSetError(MS_WEBERR, "This script should be referenced with a METHOD of GET or METHOD of POST.", "loadParams()");
       return -1;
     }
   }
@@ -446,6 +462,10 @@ cgiRequestObj *msAllocCgiObj()
   request->postrequest = NULL;
   request->httpcookiedata = NULL;
 
+  request->path_info = NULL;
+  request->api_path = NULL;
+  request->api_path_length = 0;
+
   return request;
 }
 
@@ -464,5 +484,11 @@ void msFreeCgiObj(cgiRequestObj *request)
   request->postrequest = NULL;
   request->httpcookiedata = NULL;
 
+  if(request->api_path) {
+    msFreeCharArray(request->api_path, request->api_path_length);
+    request->api_path = NULL;
+    request->api_path_length = 0;
+  }
+
   msFree(request);
 }
