@@ -4,6 +4,7 @@
  * Project:  MapServer
  * Purpose:  AGG template library types.
  * Author:   John Novak (jnovak@novacell.com)
+ * Author:   Thomas Bonfort (tbonfort@terriscope.fr)
  *
  ******************************************************************************
  * Copyright (c) 1996-2007 Regents of the University of Minnesota.
@@ -27,7 +28,7 @@
  * DEALINGS IN THE SOFTWARE.
  ******************************************************************************/
 
-#include "renderers/agg/include/agg_rendering_buffer.h"
+#include "renderers/agg/include/agg_path_storage.h"
 
 /*
  * interface to a shapeObj representing lines, providing the functions
@@ -36,7 +37,7 @@
 class line_adaptor
 {
 public:
-  line_adaptor(shapeObj *shape):s(shape) {
+  explicit line_adaptor(shapeObj *shape):s(shape) {
     m_line=s->line; /*first line*/
     m_point=m_line->point; /*current vertex is first vertex of first line*/
     m_lend=&(s->line[s->numlines]); /*pointer to after last line*/
@@ -75,7 +76,7 @@ public:
 
     return vertex(x,y); /*this will return the first point of the next line*/
   }
-private:
+protected:
   shapeObj *s;
   lineObj *m_line, /*current line pointer*/
           *m_lend; /*points to after the last line*/
@@ -83,28 +84,11 @@ private:
            *m_pend; /*points to after last point of current line*/
 };
 
-class offset_line_adaptor: public line_adaptor
-{
-public:
-  offset_line_adaptor(shapeObj *shape, double ox, double oy):line_adaptor(shape),ox(ox),oy(oy) {
-  }
-
-  unsigned vertex(double* x, double* y) {
-    unsigned ret = line_adaptor::vertex(x,y);
-    *x+=ox;
-    *y+=oy;
-    return ret;
-  }
-private:
-  double ox,oy;
-};
-
-
 
 class polygon_adaptor
 {
 public:
-  polygon_adaptor(shapeObj *shape):s(shape),m_stop(false) {
+  explicit polygon_adaptor(shapeObj *shape):s(shape) {
     m_line=s->line; /*first lines*/
     m_point=m_line->point; /*first vertex of first line*/
     m_lend=&(s->line[s->numlines]); /*pointer to after last line*/
@@ -154,33 +138,15 @@ public:
      * of the shape. return the command to stop processing this shape*/
     return mapserver::path_cmd_stop;
   }
-private:
+protected:
   shapeObj *s;
-  double ox,oy;
+  double ox = 0.0;
+  double oy = 0.0;
   lineObj *m_line, /*pointer to current line*/
           *m_lend; /*pointer to after last line of the shape*/
   pointObj *m_point, /*pointer to current vertex*/
            *m_pend; /*pointer to after last vertex of current line*/
-  bool m_stop; /*should next call return stop command*/
+  bool m_stop = false; /*should next call return stop command*/
 };
 
-class offset_polygon_adaptor: public polygon_adaptor
-{
-public:
-  offset_polygon_adaptor(shapeObj *shape, double ox, double oy):polygon_adaptor(shape),ox(ox),oy(oy) {
-  }
-
-  unsigned vertex(double* x, double* y) {
-    unsigned ret = polygon_adaptor::vertex(x,y);
-    *x+=ox;
-    *y+=oy;
-    return ret;
-  }
-private:
-  double ox,oy;
-};
-
-
-
-
-
+mapserver::path_storage imageVectorSymbol(symbolObj *);
