@@ -56,6 +56,7 @@
 #endif
 
 #include <cpl_string.h>
+#include "cpl_conv.h"
 #include <gdal.h>
 
 #include "fontcache.h"
@@ -407,12 +408,14 @@ int renderTileCairo(imageObj *img, imageObj *tile, double x, double y)
   return MS_SUCCESS;
 }
 
-int renderGlyphs2Cairo(imageObj *img, textPathObj *tp, colorObj *c, colorObj *oc, int ow, int isMarker) {
+int renderGlyphs2Cairo(imageObj *img, const textSymbolObj *ts, colorObj *c, colorObj *oc, int ow, int isMarker) {
+  const textPathObj *tp = ts->textpath;
   cairo_renderer *r = CAIRO_RENDERER(img);
   cairoCacheData *cache = MS_IMAGE_RENDERER_CACHE(img);
   cairoFaceCache *cairo_face = NULL;
   FT_Face prevface = NULL;
   int g;
+  (void)isMarker;
 
   cairo_set_font_size(r->cr,MS_NINT(tp->glyph_size * 96.0/72.0));
   for(g=0;g<tp->numglyphs;g++) {
@@ -475,6 +478,17 @@ imageObj* createImageCairo(int width, int height, outputFormatObj *format,colorO
                      _stream_write_fn,
                      r->outputStream,
                      width,height);
+#if CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1,15,10)
+      {
+          const char *msPDFCreationDate = CPLGetConfigOption("MS_PDF_CREATION_DATE", NULL);
+          if( msPDFCreationDate )
+          {
+              cairo_pdf_surface_set_metadata (r->surface,
+                                              CAIRO_PDF_METADATA_CREATE_DATE,
+                                              msPDFCreationDate);
+          }
+      }
+#endif
     } else if(!strcasecmp(format->driver,"cairo/svg")) {
       r->outputStream = (bufferObj*)malloc(sizeof(bufferObj));
       msBufferInit(r->outputStream);
@@ -643,8 +657,9 @@ static void msTransformToGeospatialPDF(imageObj *img, mapObj *map, cairo_rendere
   msFree(pszTmpFilename);
 }
 
-int saveImageCairo(imageObj *img, mapObj *map, FILE *fp, outputFormatObj *format)
+int saveImageCairo(imageObj *img, mapObj *map, FILE *fp, outputFormatObj *format_unused)
 {
+  (void)format_unused;
   cairo_renderer *r = CAIRO_RENDERER(img);
   if(!strcasecmp(img->format->driver,"cairo/pdf") || !strcasecmp(img->format->driver,"cairo/svg")) {
     cairo_surface_finish (r->surface);
@@ -659,8 +674,9 @@ int saveImageCairo(imageObj *img, mapObj *map, FILE *fp, outputFormatObj *format
   return MS_SUCCESS;
 }
 
-unsigned char* saveImageBufferCairo(imageObj *img, int *size_ptr, outputFormatObj *format)
+unsigned char* saveImageBufferCairo(imageObj *img, int *size_ptr, outputFormatObj *format_unused)
 {
+  (void)format_unused;
   cairo_renderer *r = CAIRO_RENDERER(img);
   unsigned char *data;
   assert(!strcasecmp(img->format->driver,"cairo/pdf") || !strcasecmp(img->format->driver,"cairo/svg"));
@@ -701,6 +717,7 @@ int renderEllipseSymbolCairo(imageObj *img, double x, double y, symbolObj *symbo
 
 int startLayerVectorCairo(imageObj *img, mapObj *map, layerObj *layer)
 {
+  (void)map;
   if(layer->compositer && layer->compositer->opacity<100) {
     cairo_renderer *r = CAIRO_RENDERER(img);
     cairo_push_group (r->cr);
@@ -710,6 +727,7 @@ int startLayerVectorCairo(imageObj *img, mapObj *map, layerObj *layer)
 
 int closeLayerVectorCairo(imageObj *img, mapObj *map, layerObj *layer)
 {
+  (void)map;
   if(layer->compositer && layer->compositer->opacity<100) {
     cairo_renderer *r = CAIRO_RENDERER(img);
     cairo_pop_group_to_source (r->cr);
@@ -720,11 +738,17 @@ int closeLayerVectorCairo(imageObj *img, mapObj *map, layerObj *layer)
 
 int startLayerRasterCairo(imageObj *img, mapObj *map, layerObj *layer)
 {
+  (void)img;
+  (void)map;
+  (void)layer;
   return MS_SUCCESS;
 }
 
 int closeLayerRasterCairo(imageObj *img, mapObj *map, layerObj *layer)
 {
+  (void)img;
+  (void)map;
+  (void)layer;
   return MS_SUCCESS;
 }
 
@@ -890,7 +914,6 @@ void freeSVGCache(symbolObj *s) {
 #ifdef USE_SVG_CAIRO
       svg_cairo_destroy(cache->svgc);
 #else
-      rsvg_handle_close(cache->svgc, NULL);
   #if LIBRSVG_CHECK_VERSION(2,35,0)
       g_object_unref(cache->svgc);
   #else
@@ -927,6 +950,7 @@ int freeSymbolCairo(symbolObj *s)
 
 int initializeRasterBufferCairo(rasterBufferObj *rb, int width, int height, int mode)
 {
+  (void)mode;
   rb->type = MS_BUFFER_BYTE_RGBA;
   rb->width = width;
   rb->height = height;
@@ -980,15 +1004,42 @@ int msPreloadSVGSymbol(symbolObj *symbol)
   }
 #else
   {
-    RsvgDimensionData dim;
     cache->svgc = rsvg_handle_new_from_file(symbol->full_pixmap_path,NULL);
     if(!cache->svgc) {
       msSetError(MS_RENDERERERR,"failed to load svg file %s", "msPreloadSVGSymbol()", symbol->full_pixmap_path);
       return MS_FAILURE;
     }
+#if LIBRSVG_CHECK_VERSION(2,46,0)
+    /* rsvg_handle_get_dimensions_sub() is deprecated since librsvg 2.46 */
+    /* It seems rsvg_handle_get_intrinsic_dimensions() is the best equivalent */
+    /* when the <svg> root node includes a width and height attributes in pixels */
+    gboolean has_width = FALSE;
+    RsvgLength width = {0, RSVG_UNIT_PX};
+    gboolean has_height = FALSE;
+    RsvgLength height = {0, RSVG_UNIT_PX};
+    rsvg_handle_get_intrinsic_dimensions(cache->svgc,
+                                         &has_width, &width,
+                                         &has_height, &height,
+                                         NULL, NULL);
+    if( has_width && width.unit == RSVG_UNIT_PX &&
+        has_height && height.unit == RSVG_UNIT_PX )
+    {
+        symbol->sizex = width.length;
+        symbol->sizey = height.length;
+    }
+    else
+    {
+        RsvgRectangle ink_rect = { 0, 0, 0, 0 };
+        rsvg_handle_get_geometry_for_element (cache->svgc, NULL, &ink_rect, NULL, NULL);
+        symbol->sizex = ink_rect.width;
+        symbol->sizey = ink_rect.height;
+    }
+#else
+    RsvgDimensionData dim;
     rsvg_handle_get_dimensions_sub (cache->svgc, &dim, NULL);
     symbol->sizex = dim.width;
     symbol->sizey = dim.height;
+#endif
   }
 #endif
 
