@@ -906,6 +906,7 @@ int selectClusterShape(layerObj* layer, long shapeindex)
     ++i;
     current = current->next;
   }
+  assert(current);
 
   current->next = current->siblings;
   layerinfo->current = current;
@@ -997,7 +998,7 @@ int RebuildClusters(layerObj *layer, int isQuery)
   int status;
   clusterInfo* current;
   int depth;
-  char *pszProcessing;
+  const char *pszProcessing;
 #ifdef USE_CLUSTER_EXTERNAL
   int layerIndex;
 #endif
@@ -1385,7 +1386,8 @@ int msClusterLayerClose(layerObj *layer)
 
 #ifndef USE_CLUSTER_EXTERNAL
   /* switch back to the source layer vtable */
-  msInitializeVirtualTable(layer);
+  if( msInitializeVirtualTable(layer) != MS_SUCCESS )
+      return MS_FAILURE;
 #endif
 
   return MS_SUCCESS;
@@ -1480,12 +1482,14 @@ int msClusterLayerInitItemInfo(layerObj *layer)
 /* Execute a query for this layer */
 int msClusterLayerWhichShapes(layerObj *layer, rectObj rect, int isQuery)
 {
+  (void)rect;
   /* rebuild the cluster database */
   return RebuildClusters(layer, isQuery);
 }
 
 static int prepareShape(layerObj* layer, msClusterLayerInfo* layerinfo, clusterInfo* current, shapeObj* shape)
 {
+  (void)layer;
   if (msCopyShape(&(current->shape), shape) != MS_SUCCESS) {
     msSetError(MS_SHPERR, "Cannot retrieve inline shape. There some problem with the shape", "msClusterLayerNextShape()");
     return MS_FAILURE;
@@ -1577,6 +1581,10 @@ int msClusterLayerGetNumFeatures(layerObj *layer)
 static int msClusterLayerGetAutoStyle(mapObj *map, layerObj *layer, classObj *c,
                                       shapeObj* shape)
 {
+  (void)map;
+  (void)layer;
+  (void)c;
+  (void)shape;
   /* TODO */
   return MS_SUCCESS;
 }
@@ -1665,6 +1673,7 @@ int msClusterLayerOpen(layerObj *layer)
     if (msInitializeVirtualTable(layer) != MS_SUCCESS)
       return MS_FAILURE;
   }
+  assert(layer->vtable);
   msClusterLayerCopyVirtualTable(layer->vtable);
 
   if (msCopyLayer(&layerinfo->srcLayer, layer) != MS_SUCCESS)
@@ -1689,6 +1698,7 @@ int msClusterLayerOpen(layerObj *layer)
 
 int msClusterLayerTranslateFilter(layerObj *layer, expressionObj *filter, char *filteritem)
 {
+  (void)filter;
   msClusterLayerInfo* layerinfo = layer->layerinfo;
 
   if (!layerinfo) {
@@ -1728,12 +1738,14 @@ int msClusterLayerGetAutoProjection(layerObj *layer, projectionObj* projection)
 
 int msClusterLayerGetPaging(layerObj *layer)
 {
+  (void)layer;
   return MS_FALSE;
 }
 
 void msClusterLayerEnablePaging(layerObj *layer, int value)
 {
-  return;
+  (void)layer;
+  (void)value;
 }
 
 void msClusterLayerCopyVirtualTable(layerVTableObj* vtable)
