@@ -674,7 +674,7 @@ int msLoadMapContextGeneral(mapObj *map, CPLXMLNode *psGeneral,
   /* Projection */
   pszValue = (char*)CPLGetXMLValue(psGeneral,
                                    "BoundingBox.SRS", NULL);
-  if(pszValue != NULL) {
+  if(pszValue != NULL && !EQUAL(pszValue, "(null)")) {
     if(strncasecmp(pszValue, "AUTO:", 5) == 0) {
       pszProj = msStrdup(pszValue);
     } else {
@@ -1148,9 +1148,6 @@ int msLoadMapContext(mapObj *map, char *filename, int unique_layer_names)
   if( psRoot == NULL ) {
     msSetError( MS_MAPCONTEXTERR, "Invalid XML file (%s)",
                 "msLoadMapContext()", filename );
-    if(psRoot != NULL)
-      CPLDestroyXMLNode(psRoot);
-
     return MS_FAILURE;
   }
 
@@ -1313,8 +1310,8 @@ int msWriteMapContext(mapObj *map, FILE *stream)
 #if defined(USE_WMS_LYR)
   const char * version;
   char *pszEPSG;
-  char * tabspace=NULL, *pszChar,*pszSLD=NULL,*pszURL,*pszSLD2=NULL;
-  char *pszStyle, *pszStyleItem, *pszSLDBody;
+  char * tabspace=NULL, *pszChar,*pszSLD=NULL,*pszSLD2=NULL;
+  char *pszStyleItem, *pszSLDBody;
   char *pszEncodedVal;
   int i, nValue, nVersion=OWS_VERSION_NOTSET;
   /* Dimension element */
@@ -1652,13 +1649,13 @@ int msWriteMapContext(mapObj *map, FILE *stream)
       /* Format */
       if(msLookupHashTable(&(GET_LAYER(map, i)->metadata),"wms_formatlist")==NULL &&
           msLookupHashTable(&(GET_LAYER(map, i)->metadata),"wms_format")==NULL) {
-        pszURL = NULL;
+        char* pszURL;
         if(GET_LAYER(map, i)->connection)
           pszURL = msStrdup( GET_LAYER(map, i)->connection );
         else
           pszURL = msStrdup( "" );
-        pszValueMod = pszURL;
-        pszValueMod = strstr( pszValueMod, "FORMAT=" );
+
+        pszValueMod = strstr( pszURL, "FORMAT=" );
         if( pszValueMod ) {
           pszValueMod += 7;
           pszChar = strchr(pszValueMod, '&');
@@ -1672,8 +1669,7 @@ int msWriteMapContext(mapObj *map, FILE *stream)
             msFree(pszEncodedVal);
           }
         }
-        if(pszURL)
-          free(pszURL);
+        free(pszURL);
       } else {
         char **papszFormats;
         int numFormats, nForm;
@@ -1714,14 +1710,14 @@ int msWriteMapContext(mapObj *map, FILE *stream)
           pszValue ++;
       if(pszValue == NULL || strlen(pszValue) < 1) {
         /* Check if the style is in the connection URL */
-        pszURL = "";
+        char* pszURL;
         if(GET_LAYER(map, i)->connection)
           pszURL = msStrdup( GET_LAYER(map, i)->connection );
         else
           pszURL = msStrdup( "" );
-        pszValueMod = pszURL;
+
         /* Grab the STYLES in the URL */
-        pszValueMod = strstr( pszValueMod, "STYLES=" );
+        pszValueMod = strstr( pszURL, "STYLES=" );
         if( pszValueMod ) {
           pszValueMod += 7;
           pszChar = strchr(pszValueMod, '&');
@@ -1791,10 +1787,7 @@ int msWriteMapContext(mapObj *map, FILE *stream)
             pszSLD2 = NULL;
           }
         }
-        if(pszURL) {
-          free(pszURL);
-          pszURL = NULL;
-        }
+        free(pszURL);
       } else {
         const char* pszCurrent;
         /* If the style information is not in the connection URL, */
@@ -1806,12 +1799,15 @@ int msWriteMapContext(mapObj *map, FILE *stream)
         msIO_fprintf( stream, "      <StyleList>\n");
         /* Loop in each style in the style list */
         while(pszValue != NULL) {
-          pszStyle = msStrdup(pszValue);
+          char* pszStyle = msStrdup(pszValue);
           pszChar = strchr(pszStyle, ',');
           if(pszChar != NULL)
             pszStyle[pszChar - pszStyle] = '\0';
-          if(strcasecmp(pszStyle, "") == 0)
+          if(pszStyle[0] == '\0')
+          {
+            msFree(pszStyle);
             continue;
+          }
 
           if(pszCurrent && (strcasecmp(pszStyle, pszCurrent) == 0))
             msIO_fprintf( stream,"        <Style current=\"1\">\n" );
@@ -1883,7 +1879,7 @@ int msWriteMapContext(mapObj *map, FILE *stream)
 
           msIO_fprintf( stream,"        </Style>\n" );
 
-          free(pszStyle);
+          msFree(pszStyle);
           pszValue = strchr(pszValue, ',');
           if(pszValue)
             pszValue++;
