@@ -53,21 +53,21 @@
  * Copy a projectionObj while adding additional arguments              *
  **********************************************************************/
 
-int msCopyProjectionExtended(projectionObj *dst, projectionObj *src, char ** args, int num_args)
+int msCopyProjectionExtended(projectionObj *dst, const projectionObj *src, char ** args, int num_args)
 {
-
-  int i;
-
   MS_COPYSTELEM(numargs);
   MS_COPYSTELEM(gt);
   MS_COPYSTELEM(automatic);
 
-  for (i = 0; i < dst->numargs; i++) {
+  for (int i = 0; i < dst->numargs; i++) {
     /* Our destination consists of unallocated pointers */
     dst->args[i] = msStrdup(src->args[i]);
   }
-  for(i=0 ; i< num_args; i++) {
-    dst->args[dst->numargs++] = msStrdup(args[i]);
+  if( args )
+  {
+      for(int i=0 ; i< num_args; i++) {
+        dst->args[dst->numargs++] = msStrdup(args[i]);
+      }
   }
   msProjectionInheritContextFrom(dst, src);
   if (dst->numargs != 0) {
@@ -84,7 +84,7 @@ int msCopyProjectionExtended(projectionObj *dst, projectionObj *src, char ** arg
  * Copy a projectionObj                                                *
  **********************************************************************/
 
-int msCopyProjection(projectionObj *dst, projectionObj *src)
+int msCopyProjection(projectionObj *dst, const projectionObj *src)
 {
   return msCopyProjectionExtended(dst,src,NULL,0);
 }
@@ -94,7 +94,7 @@ int msCopyProjection(projectionObj *dst, projectionObj *src)
  *                                                                     *
  * Copy a lineObj, using msCopyPoint()                                 *
  **********************************************************************/
-int msCopyLine(lineObj *dst, lineObj *src)
+int msCopyLine(lineObj *dst, const lineObj *src)
 {
 
   int i;
@@ -141,7 +141,7 @@ int msCopyShapeObj(shapeObj *dst, shapeObj *src) {
  * Copy an itemObj                                                     *
  **********************************************************************/
 
-int msCopyItem(itemObj *dst, itemObj *src)
+int msCopyItem(itemObj *dst, const itemObj *src)
 {
 
   MS_COPYSTRING(dst->name, src->name);
@@ -159,7 +159,7 @@ int msCopyItem(itemObj *dst, itemObj *src)
  * Copy a hashTableObj, using msInsertHashTable()                      *
  **********************************************************************/
 
-int msCopyHashTable(hashTableObj *dst, hashTableObj *src)
+int msCopyHashTable(hashTableObj *dst, const hashTableObj *src)
 {
   const char *key=NULL;
   while (1) {
@@ -178,7 +178,7 @@ int msCopyHashTable(hashTableObj *dst, hashTableObj *src)
  * Copy a fontSetObj, using msCreateHashTable() and msCopyHashTable()  *
  **********************************************************************/
 
-int msCopyFontSet(fontSetObj *dst, fontSetObj *src, mapObj *map)
+int msCopyFontSet(fontSetObj *dst, const fontSetObj *src, mapObj *map)
 {
 
   MS_COPYSTRING(dst->filename, src->filename);
@@ -201,7 +201,7 @@ int msCopyFontSet(fontSetObj *dst, fontSetObj *src, mapObj *map)
  * Copy an expressionObj, but only its string, type and flags          *
  **********************************************************************/
 
-int msCopyExpression(expressionObj *dst, expressionObj *src)
+int msCopyExpression(expressionObj *dst, const expressionObj *src)
 {
   if((dst->type == MS_REGEX) && dst->compiled) ms_regfree(&(dst->regex));
   dst->compiled = MS_FALSE;
@@ -219,7 +219,7 @@ int msCopyExpression(expressionObj *dst, expressionObj *src)
  * Copy a joinObj                                                      *
  **********************************************************************/
 
-int msCopyJoin(joinObj *dst, joinObj *src)
+int msCopyJoin(joinObj *dst, const joinObj *src)
 {
   MS_COPYSTRING(dst->name, src->name);
 
@@ -253,7 +253,7 @@ int msCopyJoin(joinObj *dst, joinObj *src)
  * Copy a queryMapObj, using msCopyColor()                             *
  **********************************************************************/
 
-int msCopyQueryMap(queryMapObj *dst, queryMapObj *src)
+int msCopyQueryMap(queryMapObj *dst, const queryMapObj *src)
 {
   MS_COPYSTELEM(height);
   MS_COPYSTELEM(width);
@@ -271,7 +271,7 @@ int msCopyQueryMap(queryMapObj *dst, queryMapObj *src)
  * Copy a labelLeaderObj, using msCopyStyle()                          *
  **********************************************************************/
 
-int msCopyLabelLeader(labelLeaderObj *dst, labelLeaderObj *src)
+int msCopyLabelLeader(labelLeaderObj *dst, const labelLeaderObj *src)
 {
   int i;
   assert(dst && src);
@@ -311,7 +311,7 @@ int msCopyLabelLeader(labelLeaderObj *dst, labelLeaderObj *src)
  * Copy a labelObj, using msCopyColor() and msCopyStyle()              *
  **********************************************************************/
 
-int msCopyLabel(labelObj *dst, labelObj *src)
+int msCopyLabel(labelObj *dst, const labelObj *src)
 {
   int i;
 
@@ -414,6 +414,9 @@ int msCopyLabel(labelObj *dst, labelObj *src)
     dst->leader = NULL;
   }
 
+  MS_COPYSTELEM(sizeunits);
+  MS_COPYSTELEM(scalefactor);
+
   return MS_SUCCESS;
 }
 
@@ -424,10 +427,9 @@ int msCopyLabel(labelObj *dst, labelObj *src)
  * msCopyHashTable()                                                   *
  **********************************************************************/
 
-int msCopyWeb(webObj *dst, webObj *src, mapObj *map)
+int msCopyWeb(webObj *dst, const webObj *src, mapObj *map)
 {
 
-  MS_COPYSTRING(dst->log, src->log);
   MS_COPYSTRING(dst->imagepath, src->imagepath);
   MS_COPYSTRING(dst->imageurl, src->imageurl);
   dst->map = map;
@@ -441,8 +443,6 @@ int msCopyWeb(webObj *dst, webObj *src, mapObj *map)
   MS_COPYSTRING(dst->empty, src->empty);
   MS_COPYSTRING(dst->error, src->error);
 
-  MS_COPYRECT(&(dst->extent), &(src->extent));
-
   MS_COPYSTELEM(minscaledenom);
   MS_COPYSTELEM(maxscaledenom);
   MS_COPYSTRING(dst->mintemplate, src->mintemplate);
@@ -468,7 +468,7 @@ int msCopyWeb(webObj *dst, webObj *src, mapObj *map)
  * Copy a styleObj, using msCopyColor()                                *
  **********************************************************************/
 
-int msCopyStyle(styleObj *dst, styleObj *src)
+int msCopyStyle(styleObj *dst, const styleObj *src)
 {
   int i;
 
@@ -483,7 +483,6 @@ int msCopyStyle(styleObj *dst, styleObj *src)
 
   MS_COPYCOLOR(&(dst->color), &(src->color));
   MS_COPYCOLOR(&(dst->outlinecolor),&(src->outlinecolor));
-  MS_COPYCOLOR(&(dst->backgroundcolor), &(src->backgroundcolor));
 
   MS_COPYCOLOR(&(dst->mincolor), &(src->mincolor));
   MS_COPYCOLOR(&(dst->maxcolor), &(src->maxcolor));
@@ -496,6 +495,7 @@ int msCopyStyle(styleObj *dst, styleObj *src)
   MS_COPYSTELEM(gap);
   MS_COPYSTELEM(linejoin);
   MS_COPYSTELEM(linejoinmaxsize);
+  MS_COPYSTELEM(antialiased);
   MS_COPYSTELEM(linecap);
   MS_COPYSTELEM(symbol);
   MS_COPYSTELEM(size);
@@ -520,6 +520,9 @@ int msCopyStyle(styleObj *dst, styleObj *src)
   MS_COPYSTELEM(maxscaledenom);
   /* TODO: add copy for bindings */
 
+  MS_COPYSTELEM(sizeunits);
+  MS_COPYSTELEM(scalefactor);
+
   return MS_SUCCESS;
 }
 
@@ -530,9 +533,10 @@ int msCopyStyle(styleObj *dst, styleObj *src)
  * msCopyLabel(), msCreateHashTable(), msCopyHashTable()               *
  **********************************************************************/
 
-int msCopyClass(classObj *dst, classObj *src, layerObj *layer)
+int msCopyClass(classObj *dst, const classObj *src, layerObj *layer_unused)
 {
   int i, return_value;
+  (void)layer_unused;
 
   return_value = msCopyExpression(&(dst->expression),&(src->expression));
   if (return_value != MS_SUCCESS) {
@@ -619,10 +623,13 @@ int msCopyClass(classObj *dst, classObj *src, layerObj *layer)
   MS_COPYSTELEM(layer);
   MS_COPYSTELEM(debug);
 
+  MS_COPYSTELEM(sizeunits);
+  MS_COPYSTELEM(scalefactor);
+
   return MS_SUCCESS;
 }
 
-int msCopyCluster(clusterObj *dst, clusterObj *src)
+int msCopyCluster(clusterObj *dst, const clusterObj *src)
 {
   int return_value;
 
@@ -649,7 +656,7 @@ int msCopyCluster(clusterObj *dst, clusterObj *src)
  * msCopyGrid()                                                        *
  **********************************************************************/
 
-int msCopyGrid(graticuleObj *dst, graticuleObj *src)
+int msCopyGrid(graticuleObj *dst, const graticuleObj *src)
 {
   MS_COPYSTELEM(dwhichlatitude);
   MS_COPYSTELEM(dwhichlongitude);
@@ -686,7 +693,7 @@ int msCopyGrid(graticuleObj *dst, graticuleObj *src)
  * make exact copies, this method might not get much use.              *
  **********************************************************************/
 
-int msCopyLabelCacheMember(labelCacheMemberObj *dst, labelCacheMemberObj *src)
+int msCopyLabelCacheMember(labelCacheMemberObj *dst, const labelCacheMemberObj *src)
 {
   int i;
 
@@ -721,7 +728,7 @@ int msCopyLabelCacheMember(labelCacheMemberObj *dst, labelCacheMemberObj *src)
  **********************************************************************/
 
 int msCopyMarkerCacheMember(markerCacheMemberObj *dst,
-                            markerCacheMemberObj *src)
+                            const markerCacheMemberObj *src)
 {
   MS_COPYSTELEM(id);
 
@@ -733,7 +740,7 @@ int msCopyMarkerCacheMember(markerCacheMemberObj *dst,
  * msCopyLabelCacheSlot()                                                  *
  **********************************************************************/
 
-int msCopyLabelCacheSlot(labelCacheSlotObj *dst, labelCacheSlotObj *src)
+int msCopyLabelCacheSlot(labelCacheSlotObj *dst, const labelCacheSlotObj *src)
 {
   int i;
 
@@ -754,7 +761,7 @@ int msCopyLabelCacheSlot(labelCacheSlotObj *dst, labelCacheSlotObj *src)
  * msCopyLabelCache()                                                  *
  **********************************************************************/
 
-int msCopyLabelCache(labelCacheObj *dst, labelCacheObj *src)
+int msCopyLabelCache(labelCacheObj *dst, const labelCacheObj *src)
 {
   int p;
   MS_COPYSTELEM(numlabels);
@@ -772,7 +779,7 @@ int msCopyLabelCache(labelCacheObj *dst, labelCacheObj *src)
  * msCopyResult()                                                      *
  **********************************************************************/
 
-int msCopyResult(resultObj *dst, resultObj *src)
+int msCopyResult(resultObj *dst, const resultObj *src)
 {
   MS_COPYSTELEM(shapeindex);
   MS_COPYSTELEM(tileindex);
@@ -786,7 +793,7 @@ int msCopyResult(resultObj *dst, resultObj *src)
  * msCopyResultCache()                                                 *
  **********************************************************************/
 
-int msCopyResultCache(resultCacheObj *dst, resultCacheObj *src)
+int msCopyResultCache(resultCacheObj *dst, const resultCacheObj *src)
 {
   int i;
   MS_COPYSTELEM(cachesize);
@@ -806,7 +813,7 @@ int msCopyResultCache(resultCacheObj *dst, resultCacheObj *src)
  * msCopyRect(), msCopyColor()                                         *
  **********************************************************************/
 
-int msCopyReferenceMap(referenceMapObj *dst, referenceMapObj *src,
+int msCopyReferenceMap(referenceMapObj *dst, const referenceMapObj *src,
                        mapObj *map)
 {
 
@@ -840,7 +847,7 @@ int msCopyReferenceMap(referenceMapObj *dst, referenceMapObj *src,
  * and msCopyLabel()                                                   *
  **********************************************************************/
 
-int msCopyScalebar(scalebarObj *dst, scalebarObj *src)
+int msCopyScalebar(scalebarObj *dst, const scalebarObj *src)
 {
 
   initScalebar(dst);
@@ -865,7 +872,6 @@ int msCopyScalebar(scalebarObj *dst, scalebarObj *src)
   MS_COPYSTELEM(status);
   MS_COPYSTELEM(position);
   MS_COPYSTELEM(transparent);
-  MS_COPYSTELEM(interlace);
   MS_COPYSTELEM(postlabelcache);
   MS_COPYSTELEM(align);
 
@@ -878,7 +884,7 @@ int msCopyScalebar(scalebarObj *dst, scalebarObj *src)
  * Copy a legendObj, using msCopyColor()                               *
  **********************************************************************/
 
-int msCopyLegend(legendObj *dst, legendObj *src, mapObj *map)
+int msCopyLegend(legendObj *dst, const legendObj *src, mapObj *map)
 {
   int return_value;
 
@@ -903,7 +909,6 @@ int msCopyLegend(legendObj *dst, legendObj *src, mapObj *map)
   MS_COPYSTELEM(width);
   MS_COPYSTELEM(position);
   MS_COPYSTELEM(transparent);
-  MS_COPYSTELEM(interlace);
   MS_COPYSTELEM(postlabelcache);
 
 #ifndef __cplusplus
@@ -916,14 +921,14 @@ int msCopyLegend(legendObj *dst, legendObj *src, mapObj *map)
   return MS_SUCCESS;
 }
 
-int msCopyScaleTokenEntry(scaleTokenEntryObj *src, scaleTokenEntryObj *dst) {
+int msCopyScaleTokenEntry(const scaleTokenEntryObj *src, scaleTokenEntryObj *dst) {
   MS_COPYSTRING(dst->value,src->value);
   MS_COPYSTELEM(minscale);
   MS_COPYSTELEM(maxscale);
   return MS_SUCCESS;
 }
 
-int msCopyScaleToken(scaleTokenObj *src, scaleTokenObj *dst) {
+int msCopyScaleToken(const scaleTokenObj *src, scaleTokenObj *dst) {
   int i;
   MS_COPYSTRING(dst->name,src->name);
   MS_COPYSTELEM(n_entries);
@@ -934,7 +939,7 @@ int msCopyScaleToken(scaleTokenObj *src, scaleTokenObj *dst) {
   return MS_SUCCESS;
 }
 
-int msCopyCompositingFilter(CompositingFilter **pdst, CompositingFilter *src) {
+int msCopyCompositingFilter(CompositingFilter **pdst, const CompositingFilter *src) {
   CompositingFilter *dst = NULL;
   if(!src) {
     *pdst = NULL;
@@ -954,7 +959,7 @@ int msCopyCompositingFilter(CompositingFilter **pdst, CompositingFilter *src) {
   return MS_SUCCESS;
 }
 
-int msCopyCompositer(LayerCompositer **ldst, LayerCompositer *src) {
+int msCopyCompositer(LayerCompositer **ldst, const LayerCompositer *src) {
   LayerCompositer *dst = NULL;
   if(!src) {
     *ldst = NULL;
@@ -987,7 +992,7 @@ int msCopyCompositer(LayerCompositer **ldst, LayerCompositer *src) {
  * As it stands, we are not copying a layer's resultcache              *
  **********************************************************************/
 
-int msCopyLayer(layerObj *dst, layerObj *src)
+int msCopyLayer(layerObj *dst, const layerObj *src)
 {
   int i, return_value;
   featureListNodeObjPtr current;
@@ -1117,7 +1122,6 @@ int msCopyLayer(layerObj *dst, layerObj *src)
   }
   msCopyHashTable(&dst->validation,&src->validation);
 
-  MS_COPYSTELEM(dump);
   MS_COPYSTELEM(debug);
 
   /* No need to copy the numprocessing member, as it is incremented by
@@ -1167,7 +1171,7 @@ int msCopyLayer(layerObj *dst, layerObj *src)
  * msCopyOutputFormat(), msCopyWeb(), msCopyReferenceMap()             *
  **********************************************************************/
 
-int msCopyMap(mapObj *dst, mapObj *src)
+int msCopyMap(mapObj *dst, const mapObj *src)
 {
   int i, return_value;
   outputFormatObj *format;
@@ -1204,10 +1208,6 @@ int msCopyMap(mapObj *dst, mapObj *src)
   }
 
   /* msCopyLabelCache(&(dst->labelcache), &(src->labelcache)); */
-  MS_COPYSTELEM(transparent);
-  MS_COPYSTELEM(interlace);
-  MS_COPYSTELEM(imagequality);
-
   MS_COPYRECT(&(dst->extent), &(src->extent));
 
   MS_COPYSTELEM(cellsize);
@@ -1244,8 +1244,7 @@ int msCopyMap(mapObj *dst, mapObj *src)
   /* set the active output format */
   MS_COPYSTRING(dst->imagetype, src->imagetype);
   format = msSelectOutputFormat( dst, dst->imagetype );
-  msApplyOutputFormat(&(dst->outputformat), format, MS_NOOVERRIDE,
-                      MS_NOOVERRIDE, MS_NOOVERRIDE );
+  msApplyOutputFormat(&(dst->outputformat), format, MS_NOOVERRIDE);
 
   return_value = msCopyProjection(&(dst->projection),&(src->projection));
   if (return_value != MS_SUCCESS) {
