@@ -31,8 +31,8 @@
 #ifndef MAPFILE_H
 #define MAPFILE_H
 
-enum MS_LEXER_STATES {MS_TOKENIZE_DEFAULT=0, MS_TOKENIZE_FILE, MS_TOKENIZE_STRING, MS_TOKENIZE_EXPRESSION, MS_TOKENIZE_URL_VARIABLE, MS_TOKENIZE_URL_STRING, MS_TOKENIZE_VALUE, MS_TOKENIZE_NAME};
-enum MS_TOKEN_SOURCES {MS_FILE_TOKENS=0, MS_STRING_TOKENS, MS_URL_TOKENS};
+enum MS_LEXER_STATES {MS_TOKENIZE_DEFAULT=0, MS_TOKENIZE_FILE, MS_TOKENIZE_STRING, MS_TOKENIZE_EXPRESSION, MS_TOKENIZE_VALUE, MS_TOKENIZE_NAME, MS_TOKENIZE_CONFIG};
+enum MS_TOKEN_SOURCES {MS_FILE_TOKENS=0, MS_STRING_TOKENS};
 
 /*
 ** Keyword definitions for the mapfiles and symbolfiles (used by lexer)
@@ -143,14 +143,6 @@ enum MS_TOKEN_SOURCES {MS_FILE_TOKENS=0, MS_STRING_TOKENS, MS_URL_TOKENS};
 
 #define SYMBOLSET 1105
 
-#define OVERLAYBACKGROUNDCOLOR 1106
-#define OVERLAYCOLOR 1107
-#define OVERLAYMAXSIZE 1108
-#define OVERLAYMINSIZE 1109
-#define OVERLAYOUTLINECOLOR 1110
-#define OVERLAYSIZE 1111
-#define OVERLAYSYMBOL 1112
-
 #define IMAGETYPE 1113
 #define IMAGEQUALITY 1114
 
@@ -171,8 +163,6 @@ enum MS_TOKEN_SOURCES {MS_FILE_TOKENS=0, MS_STRING_TOKENS, MS_URL_TOKENS};
 
 #define STYLEITEM 1124
 
-#define DUMP 1125
-
 #define TITLE 1126
 
 #define LINECAP 1127
@@ -203,10 +193,6 @@ enum MS_TOKEN_SOURCES {MS_FILE_TOKENS=0, MS_STRING_TOKENS, MS_URL_TOKENS};
 #define MINSUBDIVIDE  1147
 #define LABELFORMAT  1148
 
-#define DATAPATTERN 1150
-#define FILEPATTERN 1151
-#define TEMPLATEPATTERN 1152
-
 #define PROCESSING  1153
 
 /* The DEBUG macro is also used to request debugging output.  Redefine
@@ -269,7 +255,6 @@ enum MS_TOKEN_SOURCES {MS_FILE_TOKENS=0, MS_STRING_TOKENS, MS_URL_TOKENS};
 
 /* rfc40 label wrapping */
 #define MAXLENGTH 1210
-#define MINLENGTH 1211
 
 /* rfc44 URL configuration support */
 #define VALIDATION 1212
