@@ -31,6 +31,7 @@
 ** maplabel.c: Routines to enable text drawing, BITMAP or TRUETYPE.
 */
 
+#include <assert.h>
 #include <float.h>
 
 #include "mapserver.h"
@@ -89,7 +90,12 @@ void freeTextPath(textPathObj *tp) {
     free(tp->bounds.poly);
   }
 }
+
 void freeTextSymbol(textSymbolObj *ts) {
+    freeTextSymbolEx(ts, MS_TRUE);
+}
+
+void freeTextSymbolEx(textSymbolObj *ts, int doFreeLabel) {
   if(ts->textpath) {
     freeTextPath(ts->textpath);
     free(ts->textpath);
@@ -110,7 +116,7 @@ void freeTextSymbol(textSymbolObj *ts) {
     }
   }
   free(ts->annotext);
-  if(freeLabel(ts->label) == MS_SUCCESS) {
+  if(doFreeLabel && freeLabel(ts->label) == MS_SUCCESS) {
     free(ts->label);
   }
 }
@@ -139,11 +145,11 @@ void msCopyTextSymbol(textSymbolObj *dst, textSymbolObj *src) {
   *dst = *src;
   MS_REFCNT_INCR(src->label);
   dst->annotext = msStrdup(src->annotext);
-  if(dst->textpath) {
+  if(src->textpath) {
     dst->textpath = msSmallMalloc(sizeof(textPathObj));
     msCopyTextPath(dst->textpath,src->textpath);
   }
-  if(dst->style_bounds) {
+  if(src->style_bounds) {
     int i;
     dst->style_bounds = msSmallCalloc(src->label->numstyles, sizeof(label_bounds*));
     for(i=0; i<src->label->numstyles; i++) {
@@ -216,13 +222,13 @@ int msAddLabelGroup(mapObj *map, imageObj *image, layerObj* layer, int classinde
       rasterBufferObj rb;
       int x,y;
       memset(&rb,0,sizeof(rasterBufferObj));
-      if(UNLIKELY(MS_FAILURE == MS_IMAGE_RENDERER(maskLayer->maskimage)->getRasterBufferHandle(maskLayer->maskimage,&rb))) {
+      if(MS_UNLIKELY(MS_FAILURE == MS_IMAGE_RENDERER(maskLayer->maskimage)->getRasterBufferHandle(maskLayer->maskimage,&rb))) {
         return MS_FAILURE;
       }
       x = MS_NINT(point->x);
       y = MS_NINT(point->y);
       /* Using label repeatdistance, we might have a point with x/y below 0. See #4764 */
-      if (x >= 0 && x < rb.width && y >= 0 && y < rb.height) {
+      if (x >= 0 && x < (int)rb.width && y >= 0 && y < (int)rb.height) {
         assert(rb.type == MS_BUFFER_BYTE_RGBA);
         alphapixptr = rb.data.rgba.a+rb.data.rgba.row_step*y + rb.data.rgba.pixel_step*x;
         if(!*alphapixptr) {
@@ -262,7 +268,7 @@ int msAddLabelGroup(mapObj *map, imageObj *image, layerObj* layer, int classinde
     msPopulateTextSymbolForLabelAndString(ts,lbl,annotext,layerPtr->scalefactor,image->resolutionfactor, 1);
 
     if(annotext && *annotext && lbl->autominfeaturesize && featuresize > 0) {
-      if(UNLIKELY(MS_FAILURE == msComputeTextPath(map,ts))) {
+      if(MS_UNLIKELY(MS_FAILURE == msComputeTextPath(map,ts))) {
         freeTextSymbol(ts);
         free(ts);
         return MS_FAILURE;
@@ -350,7 +356,8 @@ int msAddLabel(mapObj *map, imageObj *image, labelObj *label, int layerindex, in
   int i;
   labelCacheSlotObj *cacheslot;
   labelCacheMemberObj *cachePtr=NULL;
-  char *annotext = NULL;
+  const char *annotext = NULL;
+  char *annotextToFree = NULL;
   layerObj *layerPtr;
   classObj *classPtr;
 
@@ -365,7 +372,10 @@ int msAddLabel(mapObj *map, imageObj *image, labelObj *label, int layerindex, in
   if(ts)
     annotext = ts->annotext;
   else if(shape)
-    annotext = msShapeGetLabelAnnotation(layerPtr,shape,label);
+  {
+    annotextToFree = msShapeGetLabelAnnotation(layerPtr,shape,label);
+    annotext = annotextToFree;
+  }
 
   if(!annotext) {
     /* check if we have a labelpnt style */
@@ -398,7 +408,8 @@ int msAddLabel(mapObj *map, imageObj *image, labelObj *label, int layerindex, in
     if (maskLayer->maskimage && MS_IMAGE_RENDERER(maskLayer->maskimage)->supports_pixel_buffer) {
       rasterBufferObj rb;
       memset(&rb, 0, sizeof (rasterBufferObj));
-      if(UNLIKELY(MS_FAILURE == MS_IMAGE_RENDERER(maskLayer->maskimage)->getRasterBufferHandle(maskLayer->maskimage, &rb))) {
+      if(MS_UNLIKELY(MS_FAILURE == MS_IMAGE_RENDERER(maskLayer->maskimage)->getRasterBufferHandle(maskLayer->maskimage, &rb))) {
+        msFree(annotextToFree);
         return MS_FAILURE;
       }
       assert(rb.type == MS_BUFFER_BYTE_RGBA);
@@ -406,7 +417,7 @@ int msAddLabel(mapObj *map, imageObj *image, labelObj *label, int layerindex, in
         int x = MS_NINT(point->x);
         int y = MS_NINT(point->y);
         /* Using label repeatdistance, we might have a point with x/y below 0. See #4764 */
-        if (x >= 0 && x < rb.width && y >= 0 && y < rb.height) {
+        if (x >= 0 && x < (int)rb.width && y >= 0 && y < (int)rb.height) {
           alphapixptr = rb.data.rgba.a+rb.data.rgba.row_step*y + rb.data.rgba.pixel_step*x;
           if(!*alphapixptr) {
             /* label point does not intersect mask */
@@ -414,9 +425,11 @@ int msAddLabel(mapObj *map, imageObj *image, labelObj *label, int layerindex, in
               freeTextSymbol(ts);
               free(ts);
             }
+            msFree(annotextToFree);
             return MS_SUCCESS;
           }
         } else {
+          msFree(annotextToFree);
           return MS_SUCCESS; /* label point does not intersect image extent, we cannot know if it intersects
                                 mask, so we discard it (#5237)*/
         }
@@ -425,16 +438,18 @@ int msAddLabel(mapObj *map, imageObj *image, labelObj *label, int layerindex, in
         for (i = 0; i < ts->textpath->numglyphs; i++) {
           int x = MS_NINT(ts->textpath->glyphs[i].pnt.x);
           int y = MS_NINT(ts->textpath->glyphs[i].pnt.y);
-          if (x >= 0 && x < rb.width && y >= 0 && y < rb.height) {
+          if (x >= 0 && x < (int)rb.width && y >= 0 && y < (int)rb.height) {
             alphapixptr = rb.data.rgba.a + rb.data.rgba.row_step * y + rb.data.rgba.pixel_step*x;
             if (!*alphapixptr) {
               freeTextSymbol(ts);
               free(ts);
+              msFree(annotextToFree);
               return MS_SUCCESS;
             }
           } else {
             freeTextSymbol(ts);
             free(ts);
+            msFree(annotextToFree);
             return MS_SUCCESS; /* label point does not intersect image extent, we cannot know if it intersects
                                   mask, so we discard it (#5237)*/
           }
@@ -442,6 +457,7 @@ int msAddLabel(mapObj *map, imageObj *image, labelObj *label, int layerindex, in
       }
     } else {
       msSetError(MS_MISCERR, "Layer (%s) references references a mask layer, but the selected renderer does not support them", "msAddLabel()", layerPtr->name);
+      msFree(annotextToFree);
       return (MS_FAILURE);
     }
   }
@@ -449,14 +465,18 @@ int msAddLabel(mapObj *map, imageObj *image, labelObj *label, int layerindex, in
   if(!ts) {
     ts = msSmallMalloc(sizeof(textSymbolObj));
     initTextSymbol(ts);
-    msPopulateTextSymbolForLabelAndString(ts,label,annotext,layerPtr->scalefactor,image->resolutionfactor, 1);
+    msPopulateTextSymbolForLabelAndString(ts,label,annotextToFree,layerPtr->scalefactor,image->resolutionfactor, 1);
+    // annotextToFree = NULL;
   }
 
   if(annotext && label->autominfeaturesize && featuresize > 0) {
     if(!ts->textpath) {
-      if(UNLIKELY(MS_FAILURE == msComputeTextPath(map,ts)))
+      if(MS_UNLIKELY(MS_FAILURE == msComputeTextPath(map,ts)))
         return MS_FAILURE;
     }
+    if(!ts->textpath) {
+      return MS_FAILURE;
+    }
     if(featuresize > (ts->textpath->bounds.bbox.maxx - ts->textpath->bounds.bbox.minx)) {
       /* feature is too big to be drawn, skip it */
       freeTextSymbol(ts);
@@ -485,14 +505,6 @@ int msAddLabel(mapObj *map, imageObj *image, labelObj *label, int layerindex, in
 
   cachePtr->layerindex = layerindex; /* so we can get back to this *raw* data if necessary */
   cachePtr->classindex = classindex;
-#ifdef include_deprecated
-  if(shape) {
-    cachePtr->shapetype = shape->type;
-  } else {
-    cachePtr->shapetype = MS_SHAPE_POINT;
-  }
-#endif
-
   cachePtr->leaderline = NULL;
   cachePtr->leaderbbox = NULL;
 
@@ -529,6 +541,7 @@ int msAddLabel(mapObj *map, imageObj *image, labelObj *label, int layerindex, in
     if(classPtr->styles != NULL) {
       if(msGetMarkerSize(map, classPtr->styles[0], &w, &h, layerPtr->scalefactor) != MS_SUCCESS)
         return(MS_FAILURE);
+      assert(point);
       cacheslot->markers[cacheslot->nummarkers].bounds.minx = (point->x - .5 * w);
       cacheslot->markers[cacheslot->nummarkers].bounds.miny = (point->y - .5 * h);
       cacheslot->markers[cacheslot->nummarkers].bounds.maxx = cacheslot->markers[cacheslot->nummarkers].bounds.minx + (w-1);
@@ -605,7 +618,7 @@ static inline int testSegmentLabelBBoxIntersection(const rectObj *leaderbbox, co
         }
       }
     } else {
-      pointObj tp1,tp2;
+      pointObj tp1 = {0},tp2 = {0};
       tp1.x = test->bbox.minx;
       tp1.y = test->bbox.miny;
       tp2.x = test->bbox.minx;
@@ -751,7 +764,7 @@ int msGetStringSize(mapObj *map, labelObj *label, int size, char *string, rectOb
   initTextSymbol(&ts);
   label->size = size;
   msPopulateTextSymbolForLabelAndString(&ts,label,msStrdup(string),1,1,0);
-  if(UNLIKELY(MS_FAILURE == msGetTextSymbolSize(map,&ts,r)))
+  if(MS_UNLIKELY(MS_FAILURE == msGetTextSymbolSize(map,&ts,r)))
     return MS_FAILURE;
   label->size = lsize;
   freeTextSymbol(&ts);
@@ -787,7 +800,6 @@ int msLoadFontSet(fontSetObj *fontset, mapObj *map)
 {
   VSILFILE *stream;
   const char* line;
-  char alias[64], file1[MS_PATH_LENGTH], file2[MS_PATH_LENGTH];
   char *path;
   char szPath[MS_MAXPATHLEN];
   int i;
@@ -822,11 +834,32 @@ int msLoadFontSet(fontSetObj *fontset, mapObj *map)
     if(line[0] == '#' || line[0] == '\n' || line[0] == '\r' || line[0] == ' ')
       continue; /* skip comments and blank lines */
 
-    sscanf(line,"%s %s", alias,  file1);
+    char alias[64];
+    snprintf(alias, sizeof(alias), "%s", line);
+    char* ptr = strpbrk(alias, " \t");
+    if( !ptr )
+        continue;
+    *ptr = '\0';
 
-    if (!(*file1) || !(*alias) || (strlen(file1) <= 0))
+    const char* file1StartPtr = line + (ptr - alias);
+    file1StartPtr ++;
+    /* Skip leading spaces */
+    while( isspace((int)*file1StartPtr) )
+        file1StartPtr ++;
+
+    if (!(*file1StartPtr) || !(*alias))
       continue;
 
+    char file1[MS_PATH_LENGTH];
+    snprintf(file1, sizeof(file1), "%s", file1StartPtr);
+    /* Remove trailing spaces */
+    ptr = file1 + strlen(file1) - 1;
+    while( ptr >= file1 && isspace((int)*ptr) )
+    {
+        *ptr = '\0';
+        --ptr;
+    }
+
     bFullPath = 0;
 #if defined(_WIN32) && !defined(__CYGWIN__)
     if (file1[0] == '\\' || (strlen(file1) > 1 && (file1[1] == ':')))
@@ -839,6 +872,7 @@ int msLoadFontSet(fontSetObj *fontset, mapObj *map)
     if(bFullPath) { /* already full path */
       msInsertHashTable(&(fontset->fonts), alias, file1);
     } else {
+      char file2[MS_PATH_LENGTH];
       snprintf(file2, sizeof(file2), "%s%s", path, file1);
       /* msInsertHashTable(fontset->fonts, alias, file2); */
 
@@ -865,9 +899,11 @@ int msLoadFontSet(fontSetObj *fontset, mapObj *map)
 
 int msGetTextSymbolSize(mapObj *map, textSymbolObj *ts, rectObj *r) {
   if(!ts->textpath) {
-    if(UNLIKELY(MS_FAILURE == msComputeTextPath(map,ts)))
+    if(MS_UNLIKELY(MS_FAILURE == msComputeTextPath(map,ts)))
       return MS_FAILURE;
   }
+  if(!ts->textpath)
+    return MS_FAILURE;
   *r = ts->textpath->bounds.bbox;
   return MS_SUCCESS;
 }
@@ -881,7 +917,7 @@ int msGetTextSymbolSize(mapObj *map, textSymbolObj *ts, rectObj *r) {
 
 pointObj get_metrics(pointObj *p, int position, textPathObj *tp, int ox, int oy, double rotation, int buffer, label_bounds *bounds)
 {
-  pointObj q = {0,0,0,0}; // initialize
+  pointObj q = {0}; // initialize
   double x1=0, y1=0, x2=0, y2=0;
   double sin_a,cos_a;
   double w, h, x, y;
@@ -971,7 +1007,7 @@ pointObj get_metrics(pointObj *p, int position, textPathObj *tp, int ox, int oy,
   }
   else {
     q.x = p->x + x1 - tp->bounds.bbox.minx;
-    q.y = p->y + y1 ;
+    q.y = p->y + y1 - tp->bounds.bbox.maxy;
 
     if(bounds) {
       /* no rotation, we only need to return a bbox */
@@ -1044,6 +1080,16 @@ int intersectLabelPolygons(lineObj *l1, rectObj *r1, lineObj *l2, rectObj *r2)
   } else {
     p2 = l2;
   }
+  (void)pnts1[0].x; (void)pnts1[0].y;
+  (void)pnts1[1].x; (void)pnts1[1].y;
+  (void)pnts1[2].x; (void)pnts1[2].y;
+  (void)pnts1[3].x; (void)pnts1[3].y;
+  (void)pnts1[4].x; (void)pnts1[4].y;
+  (void)pnts2[0].x; (void)pnts2[0].y;
+  (void)pnts2[1].x; (void)pnts2[1].y;
+  (void)pnts2[2].x; (void)pnts2[2].y;
+  (void)pnts2[3].x; (void)pnts2[3].y;
+  (void)pnts2[4].x; (void)pnts2[4].y;
 
   /* STEP 1: look for intersecting line segments */
   for(v1=1; v1<p1->numpoints; v1++)
@@ -1068,8 +1114,6 @@ int intersectLabelPolygons(lineObj *l1, rectObj *r1, lineObj *l2, rectObj *r2)
 /* For MapScript, exactly the same the msInsertStyle */
 int msInsertLabelStyle(labelObj *label, styleObj *style, int nStyleIndex)
 {
-  int i;
-
   if (!style) {
     msSetError(MS_CHILDERR, "Can't insert a NULL Style", "msInsertLabelStyle()");
     return -1;
@@ -1088,19 +1132,16 @@ int msInsertLabelStyle(labelObj *label, styleObj *style, int nStyleIndex)
     MS_REFCNT_INCR(style);
     label->numstyles++;
     return label->numstyles-1;
-  } else if (nStyleIndex >= 0 && nStyleIndex < label->numstyles) {
+  } else {
     /* Move styles existing at the specified nStyleIndex or greater */
     /* to a higher nStyleIndex */
-    for (i=label->numstyles-1; i>=nStyleIndex; i--) {
+    for (int i=label->numstyles-1; i>=nStyleIndex; i--) {
       label->styles[i+1] = label->styles[i];
     }
     label->styles[nStyleIndex]=style;
     MS_REFCNT_INCR(style);
     label->numstyles++;
     return nStyleIndex;
-  } else {
-    msSetError(MS_CHILDERR, "Invalid nStyleIndex", "insertLabelStyle()");
-    return -1;
   }
 }
 
@@ -1109,9 +1150,8 @@ int msInsertLabelStyle(labelObj *label, styleObj *style, int nStyleIndex)
  */
 int msMoveLabelStyleUp(labelObj *label, int nStyleIndex)
 {
-  styleObj *psTmpStyle = NULL;
   if (label && nStyleIndex < label->numstyles && nStyleIndex >0) {
-    psTmpStyle = (styleObj *)malloc(sizeof(styleObj));
+    styleObj* psTmpStyle = (styleObj *)malloc(sizeof(styleObj));
     initStyle(psTmpStyle);
 
     msCopyStyle(psTmpStyle, label->styles[nStyleIndex]);
@@ -1160,11 +1200,10 @@ int msMoveLabelStyleDown(labelObj *label, int nStyleIndex)
  */
 int msDeleteLabelStyle(labelObj *label, int nStyleIndex)
 {
-  int i = 0;
   if (label && nStyleIndex < label->numstyles && nStyleIndex >=0) {
     if (freeStyle(label->styles[nStyleIndex]) == MS_SUCCESS)
       msFree(label->styles[nStyleIndex]);
-    for (i=nStyleIndex; i< label->numstyles-1; i++) {
+    for (int i=nStyleIndex; i< label->numstyles-1; i++) {
       label->styles[i] = label->styles[i+1];
     }
     label->styles[label->numstyles-1] = NULL;
