@@ -30,6 +30,8 @@
 #include "mapserver.h"
 #include "mapows.h"
 
+#include "limits.h"
+
 /* This object is used by the various mapQueryXXXXX() functions. It stores
  * the total amount of shapes and their RAM footprint, when they are cached
  * in the resultCacheObj* of layers. This is the total number accross all queried
@@ -164,8 +166,7 @@ static void initQueryCache(queryCacheObj* queryCache)
  * limits allowed in map->query.max_cached_shape_count and
  * map->query.max_cached_shape_ram_amount.
  */
-static int canCacheShape(mapObj* map, queryCacheObj *queryCache,
-                         shapeObj* shape, int shape_ram_size)
+static int canCacheShape(mapObj* map, queryCacheObj *queryCache, int shape_ram_size)
 {
   if( !map->query.cache_shapes )
       return MS_FALSE;
@@ -205,7 +206,7 @@ static int addResult(mapObj* map, resultCacheObj *cache,
   int i;
   int shape_ram_size = (map->query.max_cached_shape_ram_amount > 0) ? 
                                             msGetShapeRAMSize( shape ) : 0;
-  int store_shape = canCacheShape (map, queryCache, shape, shape_ram_size);
+  int store_shape = canCacheShape (map, queryCache, shape_ram_size);
 
   if(cache->numresults == cache->cachesize) { /* just add it to the end */
     if(cache->cachesize == 0)
@@ -291,7 +292,7 @@ static int loadQueryResults(mapObj *map, FILE *stream)
 {
   int i, j, k, n=0;
 
-  if(1 != fread(&n, sizeof(int), 1, stream)) {
+  if(1 != fread(&n, sizeof(int), 1, stream) || n > INT_MAX - 1) {
     msSetError(MS_MISCERR,"failed to read query count from query file stream", "loadQueryResults()");
     return MS_FAILURE;
   }
@@ -445,8 +446,10 @@ static int loadQueryParams(mapObj *map, FILE *stream)
           map->query.shape->type = shapetype;
 
           if(fscanf(stream, "%d\n", &numlines) != 1) goto parse_error;
+          if( numlines > INT_MAX - 1 ) goto parse_error;
           for(i=0; i<numlines; i++) {
-            if(fscanf(stream, "%d\n", &numpoints) != 1 || numpoints<0) goto parse_error;
+            if(fscanf(stream, "%d\n", &numpoints) != 1 || numpoints<0 ||
+                numpoints > INT_MAX / (int)sizeof(pointObj)) goto parse_error;
 
             line.numpoints = numpoints;
             line.point = (pointObj *) msSmallMalloc(line.numpoints*sizeof(pointObj));
@@ -616,9 +619,9 @@ int msQueryByIndex(mapObj *map)
    * Usually, the row number will be used as resultindex. But when working with
    * databases and querying a single result, the row number is typically 0 and
    * thus useless as the index in the result cache. See #4926 #4076. Only shape
-   * files are considered to have consistent row numbers.
+   * files and flatgeobuf are considered to have consistent row numbers.
    */
-  if ( !(lp->connectiontype == MS_SHAPEFILE || lp->connectiontype == MS_TILED_SHAPEFILE) ) {
+  if ( !(lp->connectiontype == MS_SHAPEFILE || lp->connectiontype == MS_TILED_SHAPEFILE || lp->connectiontype == MS_FLATGEOBUF) ) {
     shape.resultindex = -1;
   }
 
@@ -806,7 +809,7 @@ int msQueryByFilter(mapObj *map)
     paging = msLayerGetPaging(lp);
     msLayerClose(lp); /* reset */
     status = msLayerOpen(lp);
-    if(status != MS_SUCCESS) goto query_error;
+    if(status != MS_SUCCESS) return MS_FAILURE;
     msLayerEnablePaging(lp, paging);
 
     /* disable driver paging */
@@ -825,7 +828,7 @@ int msQueryByFilter(mapObj *map)
       lp->filter = mergeFilters(&map->query.filter, map->query.filteritem, &old_filter, old_filteritem);      
       if(!lp->filter.string) {
 	msSetError(MS_MISCERR, "Filter merge failed, able to process query.", "msQueryByFilter()");
-        goto query_error;
+        goto restore_old_filter;
       }      
     } else {
       msCopyExpression(&lp->filter, &map->query.filter); /* apply new filter */
@@ -833,7 +836,7 @@ int msQueryByFilter(mapObj *map)
 
     /* build item list, we want *all* items, note this *also* build tokens for the layer filter */
     status = msLayerWhichItems(lp, MS_TRUE, NULL);
-    if(status != MS_SUCCESS) goto query_error;
+    if(status != MS_SUCCESS) goto restore_old_filter;
 
     search_rect = map->query.rect;
 
@@ -902,9 +905,12 @@ int msQueryByFilter(mapObj *map)
 
     status = msLayerWhichShapes(lp, search_rect, MS_TRUE);
     if(status == MS_DONE) { /* no overlap */
+      lp->filteritem = old_filteritem; /* point back to original value */
+      msCopyExpression(&lp->filter, &old_filter); /* restore old filter */
+      msFreeExpression(&old_filter);
       msLayerClose(lp);
       continue;
-    } else if(status != MS_SUCCESS) goto query_error;
+    } else if(status != MS_SUCCESS) goto restore_old_filter;
 
     lp->resultcache = (resultCacheObj *)malloc(sizeof(resultCacheObj)); /* allocate and initialize the result cache */
     initResultCache( lp->resultcache);
@@ -985,7 +991,7 @@ int msQueryByFilter(mapObj *map)
 
     msProjectDestroyReprojector(reprojector);
 
-    if(status != MS_DONE) goto query_error;
+    if(status != MS_DONE) return MS_FAILURE;
     if(!map->query.only_cache_result_count && lp->resultcache->numresults == 0) 
       msLayerClose(lp); /* no need to keep the layer open */
   } /* next layer */
@@ -996,15 +1002,16 @@ int msQueryByFilter(mapObj *map)
       return MS_SUCCESS;
   }
 
-  msSetError(MS_NOTFOUND, "No matching record(s) found.", "msQueryByFilter()");
-  return MS_FAILURE;
+  if (map->debug >= MS_DEBUGLEVEL_V) {
+      msDebug("msQueryByFilter(): No matching record(s) found.");
+  }
+  return(MS_SUCCESS);
 
-query_error:
-  // msFree(lp->filteritem);
-  // lp->filteritem = old_filteritem;
-  // msCopyExpression(&lp->filter, &old_filter); /* restore old filter */
-  // msFreeExpression(&old_filter);
-  // msLayerClose(lp);
+restore_old_filter:
+  lp->filteritem = old_filteritem;
+  msCopyExpression(&lp->filter, &old_filter); /* restore old filter */
+  msFreeExpression(&old_filter);
+  msLayerClose(lp);
   return MS_FAILURE;
 }
 
@@ -1117,6 +1124,7 @@ int msQueryByRect(mapObj *map)
 
     /* build item list, we want *all* items */
     status = msLayerWhichItems(lp, MS_TRUE, NULL);
+
     if(status != MS_SUCCESS) {
       msFreeShape(&searchshape);
       return(MS_FAILURE);
@@ -1302,8 +1310,10 @@ int msQueryByRect(mapObj *map)
       return(MS_SUCCESS);
   }
 
-  msSetError(MS_NOTFOUND, "No matching record(s) found.", "msQueryByRect()");
-  return(MS_FAILURE);
+  if (map->debug >= MS_DEBUGLEVEL_V) {
+      msDebug("msQueryByRect(): No matching record(s) found.");
+  }
+  return(MS_SUCCESS);
 }
 
 static int is_duplicate(resultCacheObj *resultcache, int shapeindex, int tileindex)
@@ -1626,8 +1636,10 @@ int msQueryByFeatures(mapObj *map)
     if(GET_LAYER(map, l)->resultcache && GET_LAYER(map, l)->resultcache->numresults > 0) return(MS_SUCCESS);
   }
 
-  msSetError(MS_NOTFOUND, "No matching record(s) found.", "msQueryByFeatures()");
-  return(MS_FAILURE);
+  if (map->debug >= MS_DEBUGLEVEL_V) {
+      msDebug("msQueryByFeatures(): No matching record(s) found.");
+  }
+  return(MS_SUCCESS);
 }
 
 /* msQueryByPoint()
@@ -1870,8 +1882,10 @@ int msQueryByPoint(mapObj *map)
       return(MS_SUCCESS);
   }
 
-  msSetError(MS_NOTFOUND, "No matching record(s) found.", "msQueryByPoint()");
-  return(MS_FAILURE);
+  if (map->debug >= MS_DEBUGLEVEL_V) {
+      msDebug("msQueryByPoint(): No matching record(s) found.");
+  }
+  return(MS_SUCCESS);
 }
 
 int msQueryByShape(mapObj *map)
@@ -2157,8 +2171,10 @@ int msQueryByShape(mapObj *map)
       return(MS_SUCCESS);
   }
 
-  msSetError(MS_NOTFOUND, "No matching record(s) found.", "msQueryByShape()");
-  return(MS_FAILURE);
+  if (map->debug >= MS_DEBUGLEVEL_V) {
+      msDebug("msQueryByShape(): No matching record(s) found.");
+  }
+  return(MS_SUCCESS);
 }
 
 /* msGetQueryResultBounds()
