@@ -79,7 +79,6 @@ int InvGeoTransform( double *gt_in, double *gt_out )
 static int
 msNearestRasterResampler( imageObj *psSrcImage, rasterBufferObj *src_rb,
                           imageObj *psDstImage, rasterBufferObj *dst_rb,
-                          int *panCMap,
                           SimpleTransformer pfnTransform, void *pCBData,
                           int debug, rasterBufferObj *mask_rb,
                           int bWrapAtLeftRight )
@@ -138,10 +137,10 @@ msNearestRasterResampler( imageObj *psSrcImage, rasterBufferObj *src_rb,
       if( MS_RENDERER_PLUGIN(psSrcImage->format) ) {
         int src_rb_off;
         rgbaArrayObj *src,*dst;
+        assert( src_rb && dst_rb );
         assert( src_rb->type == MS_BUFFER_BYTE_RGBA );
         src = &src_rb->data.rgba;
         dst = &dst_rb->data.rgba;
-        assert( src_rb && dst_rb );
         src_rb_off = nSrcX * src->pixel_step
                       + nSrcY * src->row_step;
 
@@ -301,7 +300,6 @@ static void msSourceSample( imageObj *psSrcImage, rasterBufferObj *rb,
 static int
 msBilinearRasterResampler( imageObj *psSrcImage, rasterBufferObj *src_rb,
                            imageObj *psDstImage, rasterBufferObj *dst_rb,
-                           int *panCMap,
                            SimpleTransformer pfnTransform, void *pCBData,
                            int debug, rasterBufferObj *mask_rb,
                            int bWrapAtLeftRight )
@@ -344,6 +342,13 @@ msBilinearRasterResampler( imageObj *psSrcImage, rasterBufferObj *src_rb,
         continue;
       }
 
+      /* If we are right off the source, skip this pixel */
+      nSrcX = (int) floor(x[nDstX]);
+      nSrcY = (int) floor(y[nDstX]);
+      if( nSrcX < 0 || (!bWrapAtLeftRight && nSrcX >= nSrcXSize)
+          || nSrcY  < 0 || nSrcY >= nSrcYSize )
+        continue;
+
       /*
       ** Offset to treat TL pixel corners as pixel location instead
       ** of the center.
@@ -360,11 +365,6 @@ msBilinearRasterResampler( imageObj *psSrcImage, rasterBufferObj *src_rb,
       dfRatioX2 = x[nDstX] - nSrcX;
       dfRatioY2 = y[nDstX] - nSrcY;
 
-      /* If we are right off the source, skip this pixel */
-      if( nSrcX2 < 0 || (!bWrapAtLeftRight && nSrcX >= nSrcXSize)
-          || nSrcY2 < 0 || nSrcY >= nSrcYSize )
-        continue;
-
       /* Trim in stuff one pixel off the edge */
       nSrcX = MS_MAX(nSrcX,0);
       nSrcY = MS_MAX(nSrcY,0);
@@ -521,7 +521,6 @@ msAverageSample( imageObj *psSrcImage, rasterBufferObj *src_rb,
 static int
 msAverageRasterResampler( imageObj *psSrcImage, rasterBufferObj *src_rb,
                           imageObj *psDstImage, rasterBufferObj *dst_rb,
-                          int *panCMap,
                           SimpleTransformer pfnTransform, void *pCBData,
                           int debug, rasterBufferObj *mask_rb )
 
@@ -1347,7 +1346,6 @@ int msResampleGDALToMap( mapObj *map, layerObj *layer, imageObj *image,
   imageObj   *srcImage;
   void  *pTCBData;
   void  *pACBData;
-  int         anCMap[256];
   char       **papszAlteredProcessing = NULL;
   int         nLoadImgXSize, nLoadImgYSize;
   double      dfOversampleRatio;
@@ -1703,12 +1701,12 @@ int msResampleGDALToMap( mapObj *map, layerObj *layer, imageObj *image,
     psrc_rb = &src_rb;
     memset( psrc_rb, 0, sizeof(rasterBufferObj) );
     if( srcImage->format->vtable->supports_pixel_buffer ) {
-      if(UNLIKELY(MS_FAILURE == srcImage->format->vtable->getRasterBufferHandle( srcImage, psrc_rb ))) {
+      if(MS_UNLIKELY(MS_FAILURE == srcImage->format->vtable->getRasterBufferHandle( srcImage, psrc_rb ))) {
         msFree(mask_rb);
         return -1;
       }
     } else {
-      if(UNLIKELY(MS_FAILURE == srcImage->format->vtable->initializeRasterBuffer(psrc_rb,nLoadImgXSize, nLoadImgYSize,MS_IMAGEMODE_RGBA))) {
+      if(MS_UNLIKELY(MS_FAILURE == srcImage->format->vtable->initializeRasterBuffer(psrc_rb,nLoadImgXSize, nLoadImgYSize,MS_IMAGEMODE_RGBA))) {
         msFree(mask_rb);
         return -1;
       }
@@ -1781,17 +1779,17 @@ int msResampleGDALToMap( mapObj *map, layerObj *layer, imageObj *image,
   if( EQUAL(resampleMode,"AVERAGE") )
     result =
       msAverageRasterResampler( srcImage, psrc_rb, image, rb,
-                                anCMap, msApproxTransformer, pACBData,
+                                msApproxTransformer, pACBData,
                                 layer->debug, mask_rb );
   else if( EQUAL(resampleMode,"BILINEAR") )
     result =
       msBilinearRasterResampler( srcImage, psrc_rb, image, rb,
-                                 anCMap, msApproxTransformer, pACBData,
+                                 msApproxTransformer, pACBData,
                                  layer->debug, mask_rb, bWrapAtLeftRight );
   else
     result =
       msNearestRasterResampler( srcImage, psrc_rb, image, rb,
-                                anCMap, msApproxTransformer, pACBData,
+                                msApproxTransformer, pACBData,
                                 layer->debug, mask_rb, bWrapAtLeftRight );
 
   /* -------------------------------------------------------------------- */
