@@ -112,6 +112,7 @@ int msCalculateScale(rectObj extent, int units, int width, int height, double re
 
 double msInchesPerUnit(int units, double center_lat)
 {
+  (void)center_lat;
   double lat_adj = 1.0, ipu = 1.0;
 
   switch (units) {
@@ -215,10 +216,7 @@ imageObj *msDrawScalebar(mapObj *map)
   sy = (2*VMARGIN) + MS_NINT(VSPACING*fontHeight) + fontHeight + map->scalebar.height - VSLOP;
 
   /*Ensure we have an image format representing the options for the scalebar.*/
-  msApplyOutputFormat( &format, map->outputformat,
-                       map->scalebar.transparent,
-                       map->scalebar.interlace,
-                       MS_NOOVERRIDE );
+  msApplyOutputFormat( &format, map->outputformat, map->scalebar.transparent);
 
   if(map->scalebar.transparent == MS_OFF) {
     if(!MS_VALID_COLOR(map->scalebar.imagecolor))
@@ -226,17 +224,16 @@ imageObj *msDrawScalebar(mapObj *map)
   }
   image = msImageCreate(map->scalebar.width, sy, format,
                         map->web.imagepath, map->web.imageurl, map->resolution, map->defresolution, &map->scalebar.imagecolor);
-  image->map = map;
-
-  /* drop this reference to output format */
-  msApplyOutputFormat( &format, NULL,
-                       MS_NOOVERRIDE, MS_NOOVERRIDE, MS_NOOVERRIDE );
 
   /* did we succeed in creating the image? */
   if(!image) {
     msSetError(MS_MISCERR, "Unable to initialize image.", "msDrawScalebar()");
     return NULL;
   }
+  image->map = map;
+
+  /* drop this reference to output format */
+  msApplyOutputFormat( &format, NULL, MS_NOOVERRIDE);
 
   switch(map->scalebar.align) {
     case(MS_ALIGN_LEFT):
@@ -276,14 +273,14 @@ imageObj *msDrawScalebar(mapObj *map)
         else if(MS_VALID_COLOR(map->scalebar.backgroundcolor))
           status = renderer->renderPolygon(image,&shape,&map->scalebar.backgroundcolor);
 
-        if(UNLIKELY(status == MS_FAILURE)) {
+        if(MS_UNLIKELY(status == MS_FAILURE)) {
           goto scale_cleanup;
         }
 
         if(strokeStyle.color) {
           status = renderer->renderLine(image,&shape,&strokeStyle);
 
-          if(UNLIKELY(status == MS_FAILURE)) {
+          if(MS_UNLIKELY(status == MS_FAILURE)) {
             goto scale_cleanup;
           }
         }
@@ -293,7 +290,7 @@ imageObj *msDrawScalebar(mapObj *map)
         p.x = ox + j*isx; /* + MS_NINT(fontPtr->w/2); */
         p.y = oy + map->scalebar.height + MS_NINT(VSPACING*fontHeight);
         status = msDrawLabel(map,image,p,msStrdup(label),&map->scalebar.label,1.0);
-        if(UNLIKELY(status == MS_FAILURE)) {
+        if(MS_UNLIKELY(status == MS_FAILURE)) {
           goto scale_cleanup;
         }
         state = -state;
@@ -305,7 +302,7 @@ imageObj *msDrawScalebar(mapObj *map)
       p.x = ox; /* + MS_NINT(fontPtr->w/2); */
       p.y = oy + map->scalebar.height + MS_NINT(VSPACING*fontHeight);
       status = msDrawLabel(map,image,p,msStrdup(label),&map->scalebar.label,1.0);
-      if(UNLIKELY(status == MS_FAILURE)) {
+      if(MS_UNLIKELY(status == MS_FAILURE)) {
         goto scale_cleanup;
       }
       break;
@@ -324,7 +321,7 @@ imageObj *msDrawScalebar(mapObj *map)
       points[0].x = ox;
       points[1].x = ox + isx*map->scalebar.intervals;
       status = renderer->renderLine(image,&shape,&strokeStyle);
-      if(UNLIKELY(status == MS_FAILURE)) {
+      if(MS_UNLIKELY(status == MS_FAILURE)) {
         goto scale_cleanup;
       }
 
@@ -334,7 +331,7 @@ imageObj *msDrawScalebar(mapObj *map)
       for(j=0; j<=map->scalebar.intervals; j++) {
         points[0].x = points[1].x = ox + j*isx;
         status = renderer->renderLine(image,&shape,&strokeStyle);
-        if(UNLIKELY(status == MS_FAILURE)) {
+        if(MS_UNLIKELY(status == MS_FAILURE)) {
           goto scale_cleanup;
         }
 
@@ -348,7 +345,7 @@ imageObj *msDrawScalebar(mapObj *map)
           p.x = ox + j*isx - MS_NINT((strlen(label)*fontWidth)/2.0);
         }
         status = msDrawLabel(map,image,p,msStrdup(label),&map->scalebar.label,1.0);
-        if(UNLIKELY(status == MS_FAILURE)) {
+        if(MS_UNLIKELY(status == MS_FAILURE)) {
           goto scale_cleanup;
         }
       }
@@ -361,7 +358,7 @@ imageObj *msDrawScalebar(mapObj *map)
 
 scale_cleanup:
   freeTextSymbol(&ts);
-  if(UNLIKELY(status == MS_FAILURE)) {
+  if(MS_UNLIKELY(status == MS_FAILURE)) {
     msFreeImage(image);
     return NULL;
   }
@@ -478,7 +475,7 @@ int msEmbedScalebar(mapObj *map, imageObj *img)
     if(msMaybeAllocateClassStyle(GET_LAYER(map, l)->class[0], 0)==MS_FAILURE) return MS_FAILURE;
     GET_LAYER(map, l)->class[0]->styles[0]->symbol = s;
     status = msDrawMarkerSymbol(map, img, &point, GET_LAYER(map, l)->class[0]->styles[0], 1.0);
-    if(UNLIKELY(status == MS_FAILURE)) {
+    if(MS_UNLIKELY(status == MS_FAILURE)) {
       goto embed_cleanup;
     }
   } else {
@@ -499,7 +496,7 @@ int msEmbedScalebar(mapObj *map, imageObj *img)
     }
     GET_LAYER(map,l)->class[0]->labels[0]->styles[0]->symbol = s;
     status = msAddLabel(map, img, GET_LAYER(map, l)->class[0]->labels[0], l, 0, NULL, &point, -1, NULL);
-    if(UNLIKELY(status == MS_FAILURE)) {
+    if(MS_UNLIKELY(status == MS_FAILURE)) {
       goto embed_cleanup;
     }
   }
@@ -567,25 +564,21 @@ double GetDeltaExtentsUsingScale(double scale, int units, double centerLat, int
 double Pix2Georef(int nPixPos, int nPixMin, int nPixMax,
                   double dfGeoMin, double dfGeoMax, int bULisYOrig)
 {
-  double      dfWidthGeo = 0.0;
-  int         nWidthPix = 0;
-  double      dfPixToGeo = 0.0;
   double      dfPosGeo = 0.0;
-  double      dfDeltaGeo = 0.0;
-  int         nDeltaPix = 0;
 
-  dfWidthGeo = dfGeoMax - dfGeoMin;
-  nWidthPix = nPixMax - nPixMin;
+  const double dfWidthGeo = dfGeoMax - dfGeoMin;
+  const int nWidthPix = nPixMax - nPixMin;
 
   if (dfWidthGeo > 0.0 && nWidthPix > 0) {
-    dfPixToGeo = dfWidthGeo / (double)nWidthPix;
+    const double dfPixToGeo = dfWidthGeo / (double)nWidthPix;
 
+    int nDeltaPix;
     if (!bULisYOrig)
       nDeltaPix = nPixPos - nPixMin;
     else
       nDeltaPix = nPixMax - nPixPos;
 
-    dfDeltaGeo = nDeltaPix * dfPixToGeo;
+    const double dfDeltaGeo = nDeltaPix * dfPixToGeo;
 
     dfPosGeo = dfGeoMin + dfDeltaGeo;
   }
