@@ -109,19 +109,28 @@ extern "C" {
   typedef SHPInfo * SHPHandle;
 #endif
 
+  /************************************************************************/
+  /*                          DBFInfo                                     */
+  /************************************************************************/
 
-
+/**
+ * An object containing information about a DBF file
+ *
+ */
   typedef struct {
 #ifdef SWIG
     %immutable;
 #endif
+
+    int   nRecords; ///< Number of records in the DBF
+    int   nFields; ///< Number of fields in the DBF
+
+
+#ifndef SWIG
     VSILFILE  *fp;
-
-    int   nRecords;
-
     unsigned int nRecordLength;
     int   nHeaderLength;
-    int   nFields;
+
     int   *panFieldOffset;
     int   *panFieldSize;
     int   *panFieldDecimals;
@@ -138,41 +147,39 @@ extern "C" {
 
     char  *pszStringField;
     int   nStringFieldLen;
-#ifdef SWIG
-    %mutable;
-#endif
+#endif /* not SWIG */
   } DBFInfo;
+
   typedef DBFInfo * DBFHandle;
 
   typedef enum {FTString, FTInteger, FTDouble, FTInvalid} DBFFieldType;
 
-  /* Shapefile object, no write access via scripts */
+  /************************************************************************/
+  /*                          shapefileObj                                */
+  /************************************************************************/
+
+/**
+ * An object representing a Shapefile. There is no write access to this object
+ * using MapScript.
+ */
   typedef struct {
 #ifdef SWIG
     %immutable;
 #endif
+
+    int type; ///< Shapefile type - see mapshape.h for values of type
+    int numshapes; ///< Number of shapes
+    rectObj bounds; ///< Extent of shapes
+
+#ifndef SWIG
     char source[MS_PATH_LENGTH]; /* full path to this file data */
-
-#ifndef SWIG
+    int lastshape;
+    ms_bitarray status;
+    int isopen;
     SHPHandle hSHP; /* SHP/SHX file pointer */
-#endif
-
-    int type; /* shapefile type */
-    int numshapes; /* number of shapes */
-    rectObj bounds; /* shape extent */
-
-#ifndef SWIG
     DBFHandle hDBF; /* DBF file pointer */
 #endif
 
-    int lastshape;
-
-    ms_bitarray status;
-
-    int isopen;
-#ifdef SWIG
-    %mutable;
-#endif
   } shapefileObj;
 
 #ifndef SWIG
@@ -187,12 +194,15 @@ extern "C" {
   } msTiledSHPLayerInfo;
 
   /* shapefileObj function prototypes  */
+  MS_DLL_EXPORT int msShapefileOpenHandle(shapefileObj *shpfile, const char *filename, SHPHandle hSHP, DBFHandle hDBF);
+  MS_DLL_EXPORT int msShapefileOpenVirtualFile(shapefileObj *shpfile, const char *filename, VSILFILE * fpSHP, VSILFILE * fpSHX, VSILFILE * fpDBF, int log_failures);
   MS_DLL_EXPORT int msShapefileOpen(shapefileObj *shpfile, const char *mode, const char *filename, int log_failures);
   MS_DLL_EXPORT int msShapefileCreate(shapefileObj *shpfile, char *filename, int type);
   MS_DLL_EXPORT void msShapefileClose(shapefileObj *shpfile);
   MS_DLL_EXPORT int msShapefileWhichShapes(shapefileObj *shpfile, rectObj rect, int debug);
 
   /* SHP/SHX function prototypes */
+  MS_DLL_EXPORT SHPHandle msSHPOpenVirtualFile( VSILFILE * fpSHP, VSILFILE * fpSHX );
   MS_DLL_EXPORT SHPHandle msSHPOpen( const char * pszShapeFile, const char * pszAccess );
   MS_DLL_EXPORT SHPHandle msSHPCreate( const char * pszShapeFile, int nShapeType );
   MS_DLL_EXPORT void msSHPClose( SHPHandle hSHP );
@@ -202,16 +212,11 @@ extern "C" {
   MS_DLL_EXPORT int msSHPReadPoint(SHPHandle psSHP, int hEntity, pointObj *point );
   MS_DLL_EXPORT int msSHPWriteShape( SHPHandle psSHP, shapeObj *shape );
   MS_DLL_EXPORT int msSHPWritePoint(SHPHandle psSHP, pointObj *point );
-  /* SHX reading */
-  MS_DLL_EXPORT int msSHXLoadAll( SHPHandle psSHP );
-  MS_DLL_EXPORT int msSHXLoadPage( SHPHandle psSHP, int shxBufferPage );
-  MS_DLL_EXPORT int msSHXReadOffset( SHPHandle psSHP, int hEntity );
-  MS_DLL_EXPORT int msSHXReadSize( SHPHandle psSHP, int hEntity );
-
 
   /* tiledShapefileObj function prototypes are in mapserver.h */
 
   /* XBase function prototypes */
+  MS_DLL_EXPORT DBFHandle msDBFOpenVirtualFile( VSILFILE * fp );
   MS_DLL_EXPORT DBFHandle msDBFOpen( const char * pszDBFFile, const char * pszAccess );
   MS_DLL_EXPORT void msDBFClose( DBFHandle hDBF );
   MS_DLL_EXPORT DBFHandle msDBFCreate( const char * pszDBFFile );
