@@ -142,7 +142,7 @@ files instead.
 #if defined(USE_THREAD)
 static int thread_debug = 0;
 
-static char *lock_names[] = {
+static char *const lock_names[] = {
   NULL, "PARSER", "GDAL", "ERROROBJ", "PROJ", "TTF", "POOL", "SDE",
   "ORACLE", "OWS", "LAYER_VTABLE", "IOCONTEXT", "TMPFILE", "DEBUGOBJ", "OGR", "TIME", "FRIBIDI", "WXS", "GEOS", NULL
 };
