@@ -31,7 +31,19 @@
 #include "mapserver.h"
 #include "maptree.h"
 
+#include <limits.h>
 
+#ifdef __BYTE_ORDER__
+/* GCC/clang predefined macro */
+#define bBigEndian (__BYTE_ORDER__ == __ORDER_BIG_ENDIAN__)
+#elif defined(_MSC_VER)
+/* MSVC doesn't support the C99 trick below, but all Microsoft
+   platforms are little-endian */
+#define bBigEndian false
+#else
+/* generic check */
+#define bBigEndian (((union{int in;char out;}){1}).out)
+#endif
 
 /* -------------------------------------------------------------------- */
 /*      If the following is 0.5, nodes will be split in half.  If it    */
@@ -89,16 +101,6 @@ SHPTreeHandle msSHPDiskTreeOpen(const char * pszTree, int debug)
 
   char    pabyBuf[16];
   int     i;
-  char    bBigEndian;
-
-  /* -------------------------------------------------------------------- */
-  /*  Establish the byte order on this machine.         */
-  /* -------------------------------------------------------------------- */
-  i = 1;
-  if( *((uchar *) &i) == 1 )
-    bBigEndian = MS_FALSE;
-  else
-    bBigEndian = MS_TRUE;
 
   /* -------------------------------------------------------------------- */
   /*  Initialize the info structure.              */
@@ -490,10 +492,13 @@ static void searchDiskTreeNode(SHPTreeHandle disktree, rectObj aoi, ms_bitarray
   if( fread( &numshapes, 4, 1, disktree->fp ) != 1 )
     goto error;
   if ( disktree->needswap ) SwapWord ( 4, &numshapes );
+  if( numshapes < 0 || numshapes > INT_MAX / 4 )
+    goto error;
 
   if(!msRectOverlap(&rect, &aoi)) { /* skip rest of this node and sub-nodes */
     offset += numshapes*sizeof(ms_int32) + sizeof(ms_int32);
-    fseek(disktree->fp, offset, SEEK_CUR);
+    if( fseek(disktree->fp, offset, SEEK_CUR) < 0 )
+        goto error;
     return;
   }
   if(numshapes > 0) {
@@ -511,11 +516,14 @@ static void searchDiskTreeNode(SHPTreeHandle disktree, rectObj aoi, ms_bitarray
         msSetBit(status, ids[i], 1);
     }
     free(ids);
+    ids = NULL;
   }
 
   if( fread( &numsubnodes, 4, 1, disktree->fp ) != 1 )
     goto error;
   if ( disktree->needswap ) SwapWord ( 4, &numsubnodes );
+  if( numsubnodes < 0 || numsubnodes > INT_MAX / 4 )
+    goto error;
 
   for(i=0; i<numsubnodes; i++)
     searchDiskTreeNode(disktree, aoi, status);
@@ -524,6 +532,7 @@ static void searchDiskTreeNode(SHPTreeHandle disktree, rectObj aoi, ms_bitarray
   
 error:
   msSetError(MS_IOERR, NULL, "searchDiskTreeNode()");
+  free(ids);
   return;
 }
 
@@ -596,15 +605,22 @@ treeNodeObj *readTreeNode( SHPTreeHandle disktree )
     return NULL;
   }
   if ( disktree->needswap ) SwapWord ( 4, &node->numshapes );
-  if( node->numshapes > 0 )
-    node->ids = (ms_int32 *)msSmallMalloc(sizeof(ms_int32)*node->numshapes);
-  res = fread( node->ids, node->numshapes*4, 1, disktree->fp );
-  if ( !res )
+  if ( node->numshapes < 0 || node->numshapes > INT_MAX / 4 )
   {
-    free(node->ids);
     free(node);
     return NULL;
   }
+  if( node->numshapes > 0 )
+  {
+    node->ids = (ms_int32 *)msSmallMalloc(sizeof(ms_int32)*node->numshapes);
+    res = fread( node->ids, node->numshapes*4, 1, disktree->fp );
+    if ( !res )
+    {
+      free(node->ids);
+      free(node);
+      return NULL;
+    }
+  }
   for( i=0; i < node->numshapes; i++ ) {
     if ( disktree->needswap ) SwapWord ( 4, &node->ids[i] );
   }
@@ -763,6 +779,7 @@ int msWriteTree(treeObj *tree, char *filename, int B_order)
   /*  Establish the byte order on this machine.         */
   /* -------------------------------------------------------------------- */
   i = 1;
+  /* cppcheck-suppress knownConditionTrueFalse */
   if( *((uchar *) &i) == 1 )
     mtBigEndian = MS_FALSE;
   else
