@@ -238,13 +238,13 @@ static void msUVRasterLayerInfoFree( layerObj *layer )
 
 int msUVRASTERLayerOpen(layerObj *layer)
 {
-  uvRasterLayerInfo *uvlinfo;
-
   /* If we don't have info, initialize an empty one now */
   if( layer->layerinfo == NULL )
     msUVRasterLayerInfoInitialize( layer );
+  if( layer->layerinfo == NULL )
+    return MS_FAILURE;
 
-  uvlinfo = (uvRasterLayerInfo *) layer->layerinfo;
+  uvRasterLayerInfo* uvlinfo = (uvRasterLayerInfo *) layer->layerinfo;
 
   uvlinfo->refcount = uvlinfo->refcount + 1;
 
@@ -509,10 +509,12 @@ int msUVRASTERLayerWhichShapes(layerObj *layer, rectObj rect, int isQuery)
   if( CSLFetchNameValue( layer->processing, "UV_SPACING" ) != NULL ) {
     spacing =
       atoi(CSLFetchNameValue( layer->processing, "UV_SPACING" ));
+    if( spacing == 0 )
+        spacing = 32;
   }
 
-  width = (int)ceil(layer->map->width/spacing);
-  height = (int)ceil(layer->map->height/spacing);
+  width = (int)(layer->map->width/spacing);
+  height = (int)(layer->map->height/spacing);
 
   /* Initialize our dummy map */
   MS_INIT_COLOR(map_tmp->imagecolor, 255,255,255,255);
@@ -715,7 +717,7 @@ int msUVRASTERLayerWhichShapes(layerObj *layer, rectObj rect, int isQuery)
     msDebug("msUVRASTERLayerWhichShapes(): width: %d, height: %d, cellsize: %g\n",
             width, height, map_tmp->cellsize);
 
-  if (layer->debug == 5)
+  if (layer->debug == MS_DEBUGLEVEL_VVV)
     msDebug("msUVRASTERLayerWhichShapes(): extent: %g %g %g %g\n",
             map_tmp->extent.minx, map_tmp->extent.miny,
             map_tmp->extent.maxx, map_tmp->extent.maxy);
@@ -724,7 +726,7 @@ int msUVRASTERLayerWhichShapes(layerObj *layer, rectObj rect, int isQuery)
      geotransform, used by the resampling*/
    msMapSetSize(map_tmp, width, height);
 
-  if (layer->debug == 5)
+  if (layer->debug == MS_DEBUGLEVEL_VVV)
     msDebug("msUVRASTERLayerWhichShapes(): geotransform: %g %g %g %g %g %g\n",
             map_tmp->gt.geotransform[0], map_tmp->gt.geotransform[1],
             map_tmp->gt.geotransform[2], map_tmp->gt.geotransform[3],
@@ -862,13 +864,11 @@ int msUVRASTERLayerGetShape(layerObj *layer, shapeObj *shape, resultObj *record)
                        uvlinfo->extent.minx, uvlinfo->extent.maxx, MS_FALSE);
   point.y = Pix2Georef(y, 0, uvlinfo->height-1,
                        uvlinfo->extent.miny, uvlinfo->extent.maxy, MS_TRUE);
-  if (layer->debug == 5)
+  if (layer->debug == MS_DEBUGLEVEL_VVV)
     msDebug("msUVRASTERLayerWhichShapes(): shapeindex: %ld, x: %g, y: %g\n",
             shapeindex, point.x, point.y);
 
-#ifdef USE_POINT_Z_M
   point.m = 0.0;
-#endif
 
   shape->type = MS_SHAPE_POINT;
   line.numpoints = 1;
@@ -916,7 +916,6 @@ int msUVRASTERLayerGetExtent(layerObj *layer, rectObj *extent)
   char szPath[MS_MAXPATHLEN];
   mapObj *map = layer->map;
   shapefileObj *tileshpfile;
-  int tilelayerindex = -1;
 
   if( (!layer->data || strlen(layer->data) == 0)
       && layer->tileindex == NULL) {
@@ -930,7 +929,7 @@ int msUVRASTERLayerGetExtent(layerObj *layer, rectObj *extent)
 
   /* If the layer use a tileindex, return the extent of the tileindex shapefile/referenced layer */
   if (layer->tileindex) {
-    tilelayerindex = msGetLayerIndex(map, layer->tileindex);
+    const int tilelayerindex = msGetLayerIndex(map, layer->tileindex);
     if(tilelayerindex != -1) /* does the tileindex reference another layer */
       return msLayerGetExtent(GET_LAYER(map, tilelayerindex), extent);
     else {
