@@ -366,7 +366,7 @@ static int msBuildWMSLayerURLBase(mapObj *map, layerObj *lp,
    */
   if (pszSLDBody) {
     if (strcasecmp(pszSLDBody, "AUTO") == 0) {
-      if (pszVersion && strncmp(pszVersion, "1.3.0", 5) == 0)
+      if (strncmp(pszVersion, "1.3.0", 5) == 0)
         pszSLDGenerated = msSLDGenerateSLD(map, lp->index, "1.1.0");
       else
         pszSLDGenerated = msSLDGenerateSLD(map, lp->index, NULL);
@@ -420,7 +420,7 @@ msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
 {
 #ifdef USE_WMS_LYR
   char *pszEPSG = NULL;
-  const char *pszVersion, *pszRequestParam, *pszExceptionsParam,
+  const char *pszVersion, *pszRequestParam,
         *pszSrsParamName="SRS", *pszLayer=NULL, *pszQueryLayers=NULL,
         *pszUseStrictAxisOrder;
   rectObj bbox;
@@ -626,6 +626,11 @@ msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
       the projection block matches the _srs metadata (the search for ' '
       in ows_srs is a test to see if there are multiple EPSG: codes) */
       if( lp->projection.numargs == 0 || ows_srs == NULL || (strchr(ows_srs,' ') != NULL) ) {
+        // Reproject layer extent if lp-projection is set since we might change layer projection
+        if (msProjectionsDiffer(&(map->projection), &(lp->projection))) {
+          msProjectRect(&(lp->projection),&(map->projection), &(lp->extent));
+        }
+
         if (strncasecmp(pszEPSG, "EPSG:", 5) == 0) {
           char szProj[20];
           snprintf(szProj, sizeof(szProj), "init=epsg:%s", pszEPSG+5);
@@ -809,6 +814,7 @@ msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
     else
       pszRequestParam = "feature_info";
 
+    const char* pszExceptionsParam;
     if (nVersion >= OWS_1_3_0)
       pszExceptionsParam = "XML";
     else if (nVersion >= OWS_1_1_0) /* 1.1.0 to 1.1.0 */
@@ -866,14 +872,15 @@ msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
     }
     pszRequestParam = "GetLegendGraphic";
 
-    pszExceptionsParam = msOWSLookupMetadata(&(lp->metadata),
+    /*
+    const char* pszExceptionsParam = msOWSLookupMetadata(&(lp->metadata),
                          "MO", "exceptions_format");
     if (pszExceptionsParam == NULL) {
       if (nVersion >= OWS_1_1_0 && nVersion < OWS_1_3_0)
         pszExceptionsParam = "application/vnd.ogc.se_inimage";
       else
         pszExceptionsParam = "INIMAGE";
-    }
+    }*/
 
     if (pszLayer) { /* not set in CONNECTION string */
       msSetWMSParamString(psWMSParams, "LAYER", pszLayer, MS_FALSE, nVersion);
@@ -894,7 +901,7 @@ msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
     else
       pszRequestParam = "map";
 
-    pszExceptionsParam = msOWSLookupMetadata(&(lp->metadata),
+    const char* pszExceptionsParam = msOWSLookupMetadata(&(lp->metadata),
                          "MO", "exceptions_format");
 
     if (!bIsEssential) {
@@ -992,6 +999,7 @@ int msPrepareWMSLayerRequest(int nLayerId, mapObj *map, layerObj *lp,
   int bbox_width = 0, bbox_height = 0;
   int nTimeout, bOkToMerge, bForceSeparateRequest, bCacheToDisk;
   wmsParamsObj sThisWMSParams;
+  int ret = MS_FAILURE;
 
   if (lp->connectiontype != MS_WMS)
     return MS_FAILURE;
@@ -1003,35 +1011,39 @@ int msPrepareWMSLayerRequest(int nLayerId, mapObj *map, layerObj *lp,
    * compute BBOX in that projection.
    * ------------------------------------------------------------------ */
 
-
-  if (nRequestType == WMS_GETMAP &&
-      ( msBuildWMSLayerURL(map, lp, WMS_GETMAP,
+  switch( nRequestType )
+  {
+      case WMS_GETMAP:
+          ret = msBuildWMSLayerURL(map, lp, WMS_GETMAP,
                            0, 0, 0, NULL, &bbox, &bbox_width, &bbox_height,
-                           &sThisWMSParams) != MS_SUCCESS) ) {
-    /* an error was already reported. */
-    msFreeWmsParamsObj(&sThisWMSParams);
-    return MS_FAILURE;
-  }
+                           &sThisWMSParams);
+          break;
 
-  else if (nRequestType == WMS_GETFEATUREINFO &&
-           msBuildWMSLayerURL(map, lp, WMS_GETFEATUREINFO,
+      case WMS_GETFEATUREINFO:
+          ret = msBuildWMSLayerURL(map, lp, WMS_GETFEATUREINFO,
                               nClickX, nClickY, nFeatureCount, pszInfoFormat,
                               NULL, NULL, NULL,
-                              &sThisWMSParams) != MS_SUCCESS ) {
-    /* an error was already reported. */
-    msFreeWmsParamsObj(&sThisWMSParams);
-    return MS_FAILURE;
-  } else if (nRequestType == WMS_GETLEGENDGRAPHIC &&
-             msBuildWMSLayerURL(map, lp, WMS_GETLEGENDGRAPHIC,
+                              &sThisWMSParams);
+          break;
+
+      case WMS_GETLEGENDGRAPHIC:
+          ret = msBuildWMSLayerURL(map, lp, WMS_GETLEGENDGRAPHIC,
                                 0, 0, 0, NULL,
                                 NULL, NULL, NULL,
-                                &sThisWMSParams) != MS_SUCCESS ) {
+                                &sThisWMSParams);
+          break;
+
+      default:
+          assert(FALSE);
+          break;
+  }
+
+  if( ret != MS_SUCCESS) {
     /* an error was already reported. */
     msFreeWmsParamsObj(&sThisWMSParams);
     return MS_FAILURE;
   }
 
-
   /* ------------------------------------------------------------------
    * Check if the request is empty, perhaps due to reprojection problems
    * or wms_extents restrictions.
@@ -1182,7 +1194,7 @@ int msPrepareWMSLayerRequest(int nLayerId, mapObj *map, layerObj *lp,
     if(pszHTTPCookieData == NULL || sThisWMSParams.httpcookiedata == NULL) {
       bOkToMerge = MS_FALSE;
     }
-    if(strcmp(pszHTTPCookieData, sThisWMSParams.httpcookiedata) != 0) {
+    else if(strcmp(pszHTTPCookieData, sThisWMSParams.httpcookiedata) != 0) {
       bOkToMerge = MS_FALSE;
     }
   }
