@@ -150,9 +150,6 @@ namespace mapserver
     {   
         typedef typename PathStorage::value_type value_type;
 
-        FT_Vector   v_last;
-        FT_Vector   v_control;
-        FT_Vector   v_start;
         double x1, y1, x2, y2, x3, y3;
 
         FT_Vector*  point;
@@ -172,10 +169,9 @@ namespace mapserver
             last  = outline.contours[n];
             limit = outline.points + last;
 
-            v_start = outline.points[first];
-            v_last  = outline.points[last];
+            FT_Vector v_start = outline.points[first];
 
-            v_control = v_start;
+            FT_Vector v_control = v_start;
 
             point = outline.points + first;
             tags  = outline.tags  + first;
@@ -187,6 +183,8 @@ namespace mapserver
             // check first point to determine origin
             if( tag == FT_CURVE_TAG_CONIC)
             {
+                const FT_Vector v_last  = outline.points[last];
+
                 // first point is conic control.  Yes, this happens.
                 if(FT_CURVE_TAG(outline.tags[last]) == FT_CURVE_TAG_ON)
                 {
@@ -201,8 +199,6 @@ namespace mapserver
                     // for closure
                     v_start.x = (v_start.x + v_last.x) / 2;
                     v_start.y = (v_start.y + v_last.y) / 2;
-
-                    v_last = v_start;
                 }
                 point--;
                 tags--;
@@ -624,10 +620,10 @@ namespace mapserver
                 {
                     delete [] m_face_names[0];
                     FT_Done_Face(m_faces[0]);
-                    memcpy(m_faces, 
+                    memmove(m_faces, 
                            m_faces + 1, 
                            (m_max_faces - 1) * sizeof(FT_Face));
-                    memcpy(m_face_names, 
+                    memmove(m_face_names, 
                            m_face_names + 1, 
                            (m_max_faces - 1) * sizeof(char*));
                     m_num_faces = m_max_faces - 1;
@@ -755,7 +751,7 @@ namespace mapserver
     }
 
     //------------------------------------------------------------------------
-    bool font_engine_freetype_base::char_map(FT_Encoding char_map)
+    bool font_engine_freetype_base::char_map(FT_Encoding /*char_map*/)
     {
         if(m_cur_face)
         {
